from typing import Any

__all__ = ("MapRoute", "Router", "expand_router_string", "prepare")

from celery.app.base import Celery
from celery.app.task import Task

def expand_router_string(router: str) -> Any: ...
def prepare(routes: list[Any] | tuple[Any, ...] | None) -> list[Any]: ...

class MapRoute:
    def __init__(self, map: dict[str, Any]) -> None: ...
    def __call__(
        self, name: str, *args: Any, **kwargs: Any
    ) -> dict[str, Any] | None: ...

class Router:
    def __init__(
        self,
        routes: Any = ...,
        queues: Any = ...,
        create_missing: bool = ...,
        app: Celery | None = ...,
    ) -> None: ...
    def expand_destination(self, route: dict[str, Any]) -> dict[str, Any]: ...
    def lookup_route(
        self,
        name: str,
        args: tuple[Any, ...] | None = None,
        kwargs: dict[str, Any] | None = None,
        options: dict[str, Any] | None = None,
        task_type: type[Task[Any, Any]] | None = None,
    ) -> dict[str, Any] | None: ...
    def query_router(
        self,
        router: Any,
        task: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        options: dict[str, Any],
        task_type: type[Task[Any, Any]] | None,
    ) -> dict[str, Any] | None: ...
    def route(
        self,
        options: dict[str, Any],
        name: str,
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        task_type: type[Task[Any, Any]] | None = None,
    ) -> dict[str, Any]: ...



from collections.abc import Callable
from typing import Any

import kombu
from celery.app.base import Celery
from kombu.transport.virtual import Channel
from typing_extensions import Self

__all__ = ("EventDispatcher",)

class EventDispatcher:
    DISABLED_TRANSPORTS: set[str]
    app: Celery | None
    connection: kombu.Connection | None
    hostname: str
    groups: set[str] | None
    enabled: bool
    on_enabled: Callable[[], None] | None
    on_disabled: Callable[[], None] | None

    def __init__(
        self,
        connection: kombu.Connection | None = None,
        hostname: str | None = None,
        enabled: bool = True,
        channel: Channel | None = None,
        buffer_while_offline: bool = True,
        app: Celery | None = None,
        serializer: str | None = None,
        groups: list[str] | None = None,
        delivery_mode: int | str = ...,
        buffer_group: list[str] | None = None,
        buffer_limit: int = 24,
        on_send_buffered: Callable[..., Any] | None = None,
    ) -> None: ...
    def enable(self) -> None: ...
    def disable(self) -> None: ...
    def publish(
        self,
        type: str,
        fields: dict[str, Any],
        producer: kombu.Producer,
        blind: bool = False,
        Event: Callable[..., dict[str, Any]] = ...,
        **kwargs: Any,
    ) -> None: ...
    def send(
        self,
        type: str,
        blind: bool = False,
        utcoffset: Callable[[], int] = ...,
        retry: bool = False,
        retry_policy: dict[str, Any] | None = None,
        Event: Callable[..., dict[str, Any]] = ...,
        **fields: Any,
    ) -> None: ...
    def extend_buffer(
        self,
        other: EventDispatcher,
    ) -> None: ...
    def flush(
        self,
        errors: bool = True,
        groups: bool = True,
    ) -> None: ...
    def close(self) -> None: ...
    @property
    def publisher(self) -> kombu.Producer | None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...



from typing import Any

__all__ = ("CursesMonitor", "evtop")

class CursesMonitor:
    keymap: dict[Any, Any]
    keyalias: dict[Any, Any]
    win: Any
    selected_position: int
    screen_delay: float
    background: int
    foreground: int
    greet: str
    help: str

    def __init__(self, state: Any, app: Any, keymap: Any = None) -> None: ...
    def alert(self, callback: Any, title: str | None = None) -> None: ...
    def alert_remote_control_reply(self, reply: Any) -> None: ...
    @property
    def display_height(self) -> int: ...
    def display_task_row(self, lineno: int, task: Any) -> None: ...
    @property
    def display_width(self) -> int: ...
    def draw(self) -> None: ...
    def find_position(self) -> int: ...
    def format_row(
        self, uuid: str, task: Any, worker: Any, timestamp: Any, state: Any
    ) -> str: ...
    def handle_keypress(self) -> bool | None: ...
    @property
    def help_title(self) -> str: ...
    @property
    def info_str(self) -> str: ...
    def init_screen(self) -> None: ...
    @property
    def limit(self) -> int: ...
    def move_selection(self, direction: int = 1) -> None: ...
    def move_selection_down(self) -> None: ...
    def move_selection_up(self) -> None: ...
    def nap(self) -> None: ...
    @property
    def online_str(self) -> str: ...
    def readline(self, x: int, y: int) -> str: ...
    def resetscreen(self) -> None: ...
    def revoke_selection(self) -> None: ...
    def safe_add_str(self, y: int, x: int, string: str, *args: Any) -> None: ...
    @property
    def screen_height(self) -> int: ...
    @property
    def screen_width(self) -> int: ...
    @property
    def selected_str(self) -> str: ...
    @property
    def selected_task(self) -> Any: ...
    def selection_info(self) -> None: ...
    def selection_rate_limit(self) -> None: ...
    def selection_result(self) -> None: ...
    def selection_traceback(self) -> None: ...
    @property
    def tasks(self) -> list[Any]: ...
    @property
    def workers(self) -> list[Any]: ...

def evtop(app: Any | None = None) -> None: ...



from queue import Queue
from typing import Any

from typing_extensions import Self

__all__ = ["Client", "Listener", "Pipe"]

class Connection:
    def __init__(self, _in: Queue[Any], _out: Queue[Any]) -> None: ...
    def poll(self, timeout: float = ...) -> bool: ...
    def close(self) -> None: ...

class Listener:
    def __init__(
        self,
        address: Any = ...,
        family: str | None = ...,
        backlog: int = ...,
    ) -> None: ...
    def accept(self) -> Connection: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: object) -> None: ...
    @property
    def address(self) -> Any: ...

def Client(address: Any) -> Connection: ...
def Pipe(duplex: bool = ...) -> tuple[Connection, Connection]: ...



from collections.abc import Callable
from logging import Logger
from typing import Any

from amqp.exceptions import AMQPNotImplementedError as AMQPNotImplementedError
from amqp.exceptions import RecoverableConnectionError as RecoverableConnectionError
from typing_extensions import Self
from vine import promise

__all__ = ("AbstractChannel",)

AMQP_LOGGER: Logger
IGNORED_METHOD_DURING_CHANNEL_CLOSE: str

class AbstractChannel:
    auto_decode: bool
    channel_id: int | None
    connection: Any
    is_closing: bool
    method_queue: list[Any]

    def __init__(self, connection: Any, channel_id: int | None) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...
    def close(self) -> None: ...
    def dispatch_method(
        self, method_sig: tuple[int, int], payload: bytes, content: Any
    ) -> None: ...
    def send_method(
        self,
        sig: tuple[int, int],
        format: str | None = ...,
        args: tuple[Any, ...] | None = ...,
        content: Any | None = ...,
        wait: Any | None = ...,
        callback: Callable[..., Any] | None = ...,
        returns_tuple: bool = ...,
    ) -> Any: ...
    def wait(
        self,
        method: Any,
        callback: Callable[..., Any] | None = ...,
        timeout: float | None = ...,
        returns_tuple: bool = ...,
    ) -> Any: ...

def dumps(format: str, values: tuple[Any, ...]) -> bytes: ...
def loads(format: str, buf: bytes, offset: int) -> tuple[Any, ...]: ...
def ensure_promise(p: promise | Callable[..., Any] | None) -> promise: ...



from collections.abc import Callable
from multiprocessing.context import BaseContext

__all__ = ["BoundedSemaphore", "Condition", "Event", "Lock", "RLock", "Semaphore"]

class Semaphore:
    def __init__(self, value: int = ..., ctx: BaseContext | None = ...) -> None: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args: object) -> None: ...
    def get_value(self) -> int: ...

class BoundedSemaphore(Semaphore):
    def __init__(self, value: int = ..., ctx: BaseContext | None = ...) -> None: ...

class Lock:
    def __init__(self, ctx: BaseContext | None = ...) -> None: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args: object) -> None: ...

class RLock:
    def __init__(self, ctx: BaseContext | None = ...) -> None: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args: object) -> None: ...

class Condition:
    def __init__(
        self, lock: Lock | RLock | None = ..., ctx: BaseContext | None = ...
    ) -> None: ...
    def notify(self) -> None: ...
    def notify_all(self) -> None: ...
    def wait(self, timeout: float | None = ...) -> bool: ...
    def wait_for(
        self, predicate: Callable[[], bool], timeout: float | None = ...
    ) -> bool: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args: object) -> None: ...

class Event:
    def __init__(self, ctx: BaseContext | None = ...) -> None: ...
    def is_set(self) -> bool: ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    def wait(self, timeout: float | None = ...) -> bool: ...

class Barrier:
    def __init__(
        self,
        parties: int,
        action: Callable[[], object] | None = ...,
        timeout: float | None = ...,
        ctx: BaseContext | None = ...,
    ) -> None: ...
    def wait(self, timeout: float | None = ...) -> int: ...
    def reset(self) -> None: ...
    def abort(self) -> None: ...
    @property
    def parties(self) -> int: ...
    @property
    def n_waiting(self) -> int: ...
    @property
    def broken(self) -> bool: ...



from collections import deque
from collections.abc import Callable, Iterator
from typing import Any

__all__ = ("reprstream", "saferepr")

def saferepr(
    o: Any, maxlen: int | None = ..., maxlevels: int = ..., seen: set[int] | None = ...
) -> str: ...
def reprstream(
    stack: deque[Any],
    seen: set[int] | None = ...,
    maxlevels: int = ...,
    level: int = ...,
    isinstance: Callable[[Any, Any], bool] = ...,
) -> Iterator[tuple[Any, Any]]: ...



from typing import Any

from celery.backends.base import Backend

__all__ = ("CouchBackend",)

class CouchBackend(Backend):
    host: str
    port: int
    container: str
    username: str | None
    password: str | None
    scheme: str

    @property
    def connection(self) -> Any: ...
    def __init__(self, url: str | None = None, *args: Any, **kwargs: Any) -> None: ...
    def get(self, key: str) -> Any: ...
    def set(self, key: str, value: Any) -> None: ...
    def mget(self, keys: list[str]) -> list[Any]: ...
    def delete(self, key: str) -> None: ...



import pickle
from collections.abc import Callable
from typing import Any

__all__ = [
    "DupFd",
    "ForkingPickler",
    "dump",
    "recv_handle",
    "recvfds",
    "register",
    "send_handle",
    "sendfds",
]

ForkingPickler: type[pickle.Pickler]

def dump(obj: Any, file: Any, protocol: int | None = ...) -> None: ...
def register(type: type, reduce: Callable[[Any], Any]) -> None: ...
def DupFd(fd: int) -> Any: ...
def sendfds(sock: Any, fds: list[int]) -> None: ...
def recvfds(sock: Any, size: int) -> list[int]: ...
def send_handle(conn: Any, handle: int, destination_pid: int) -> None: ...
def recv_handle(conn: Any) -> int: ...



from typing import Any, NoReturn

from kombu.asynchronous.aws.sqs.message import AsyncMessage
from vine import promise

def list_first(rs: list[Any]) -> Any | None: ...

class AsyncQueue:
    connection: Any
    url: str | None
    message_class: type[AsyncMessage]
    visibility_timeout: int | None

    def __init__(
        self,
        connection: Any = ...,
        url: str | None = ...,
        message_class: type[AsyncMessage] = ...,
    ) -> None: ...
    def _NA(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def clear(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def count_slow(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def dump(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def load(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def load_from_file(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def load_from_filename(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def load_from_s3(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def save(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def save_to_file(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def save_to_filename(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def save_to_s3(self, *args: Any, **kwargs: Any) -> NoReturn: ...
    def get_attributes(self, attributes: str = ..., callback: Any = ...) -> promise: ...
    def set_attribute(
        self, attribute: str, value: Any, callback: Any = ...
    ) -> promise: ...
    def get_timeout(self, callback: Any = ...) -> promise: ...
    def _coerce_field_value(self, key: str, type: type, response: Any) -> Any: ...
    def set_timeout(self, visibility_timeout: int, callback: Any = ...) -> promise: ...
    def _on_timeout_set(self, visibility_timeout: int | None) -> int | None: ...
    def add_permission(
        self, label: str, aws_account_id: str, action_name: str, callback: Any = ...
    ) -> promise: ...
    def remove_permission(self, label: str, callback: Any = ...) -> promise: ...
    def read(
        self,
        visibility_timeout: int | None = ...,
        wait_time_seconds: int | None = ...,
        callback: Any = ...,
    ) -> promise: ...
    def write(
        self, message: Any, delay_seconds: int | None = ..., callback: Any = ...
    ) -> promise: ...
    def write_batch(self, messages: list[Any], callback: Any = ...) -> promise: ...
    def _on_message_sent(self, orig_message: Any, new_message: Any) -> Any: ...
    def get_messages(
        self,
        num_messages: int = ...,
        visibility_timeout: int | None = ...,
        attributes: Any = ...,
        wait_time_seconds: int | None = ...,
        callback: Any = ...,
    ) -> promise: ...
    def delete_message(self, message: Any, callback: Any = ...) -> promise: ...
    def delete_message_batch(
        self, messages: list[Any], callback: Any = ...
    ) -> promise: ...
    def change_message_visibility_batch(
        self, messages: list[Any], callback: Any = ...
    ) -> promise: ...
    def delete(self, callback: Any = ...) -> promise: ...
    def count(
        self, page_size: int = ..., vtimeout: int = ..., callback: Any = ...
    ) -> promise: ...



from collections.abc import Callable
from threading import Thread
from types import TracebackType
from typing import Any
from zoneinfo import ZoneInfo

from typing_extensions import Self, override

__all__ = ("Entry", "Schedule", "Timer", "to_timestamp")

def to_timestamp(
    d: Any,
    default_timezone: ZoneInfo = ...,
    time: Callable[[], float] = ...,
) -> float: ...

class Entry:
    fun: Callable[..., Any]
    args: tuple[Any, ...] | None
    kwargs: dict[str, Any] | None
    tref: Any
    canceled: bool

    def __init__(
        self,
        fun: Callable[..., Any],
        args: tuple[Any, ...] | None = None,
        kwargs: dict[str, Any] | None = None,
    ) -> None: ...
    def __call__(self) -> Any: ...
    def __lt__(self, other: Entry) -> bool: ...
    def __gt__(self, other: Entry) -> bool: ...
    def __le__(self, other: Entry) -> bool: ...
    def __ge__(self, other: Entry) -> bool: ...
    def cancel(self) -> None: ...
    @property
    def cancelled(self) -> bool: ...

_Entry = Entry

class Schedule:
    Entry: type[_Entry]
    on_error: Callable[[Exception], None] | None

    def __init__(
        self,
        max_interval: float | None = None,
        on_error: Callable[[Exception], None] | None = None,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def __iter__(
        self,
        min: Callable[..., Any] = ...,
        nowfun: Callable[[], float] = ...,
        pop: Callable[..., Any] = ...,
        push: Callable[..., Any] = ...,
    ) -> Self: ...
    def __len__(self) -> int: ...
    def apply_entry(self, entry: _Entry) -> Any: ...
    def call_after(
        self,
        secs: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        priority: int = 0,
    ) -> _Entry: ...
    def call_at(
        self,
        eta: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        priority: int = 0,
    ) -> _Entry: ...
    def call_repeatedly(
        self,
        secs: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        priority: int = 0,
    ) -> _Entry: ...
    def cancel(self, tref: Any) -> None: ...
    def clear(self) -> None: ...
    def enter_at(
        self,
        entry: _Entry,
        eta: float | None = None,
        priority: int = 0,
        time: Callable[[], float] = ...,
    ) -> _Entry: ...
    def enter_after(
        self,
        secs: float,
        entry: _Entry,
        priority: int = 0,
        time: Callable[[], float] = ...,
    ) -> _Entry: ...
    def handle_error(
        self, exc_info: tuple[type[BaseException], BaseException, TracebackType | None]
    ) -> None: ...
    @property
    def queue(self) -> list[_Entry]: ...
    @property
    def schedule(self) -> Any: ...
    def stop(self) -> None: ...

_Schedule = Schedule

class Timer(Thread):
    Entry: type[_Entry]
    Schedule: type[_Schedule]
    running: bool
    on_tick: Callable[[float], None] | None

    def __init__(
        self,
        schedule: Any | None = None,
        on_error: Callable[[Exception], None] | None = None,
        on_tick: Callable[[float], None] | None = None,
        on_start: Callable[[Timer], None] | None = None,
        max_interval: float | None = None,
        **kwargs: Any,
    ) -> None: ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def __next__(self) -> _Entry: ...
    def call_after(
        self,
        secs: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        priority: int = 0,
    ) -> _Entry: ...
    def call_at(
        self,
        eta: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        priority: int = 0,
    ) -> _Entry: ...
    def call_repeatedly(
        self,
        secs: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        priority: int = 0,
    ) -> _Entry: ...
    def enter(
        self,
        entry: _Entry,
        eta: float,
        priority: int | None = None,
    ) -> _Entry: ...
    def enter_after(
        self,
        secs: float,
        entry: _Entry,
        priority: int = 0,
    ) -> _Entry: ...
    def exit_after(
        self,
        secs: float,
        priority: int = 10,
    ) -> None: ...
    def cancel(self, tref: Any) -> None: ...
    def clear(self) -> None: ...
    def empty(self) -> bool: ...
    def next(self) -> _Entry | None: ...
    @override
    def run(self) -> None: ...
    def stop(self) -> None: ...
    def ensure_started(self) -> None: ...
    @property
    def queue(self) -> list[Any]: ...



from typing import Any

__all__ = ("CertStore", "Certificate", "FSCertStore")

class Certificate:
    def __init__(self, cert: bytes | str) -> None: ...
    def has_expired(self) -> bool: ...
    def get_id(self) -> str: ...
    def get_pubkey(self) -> Any: ...
    def get_serial_number(self) -> int: ...
    def get_issuer(self) -> str: ...
    def verify(self, data: bytes, signature: bytes, digest: str) -> None: ...

class CertStore:
    def __init__(self) -> None: ...
    def __getitem__(self, id: str) -> Certificate: ...
    def itercerts(self) -> list[Certificate]: ...
    def add_cert(self, cert: Certificate) -> None: ...

class FSCertStore(CertStore):
    def __init__(self, path: str) -> None: ...



from typing import Any, ClassVar, Literal

from celery.app.task import Task as BaseTask
from docutils import nodes
from sphinx.domains.python import PyFunction
from sphinx.ext.autodoc import Documenter, FunctionDocumenter
from typing_extensions import override

class TaskDocumenter(FunctionDocumenter):
    objtype: ClassVar[str]
    member_order: ClassVar[int]

    @override
    @classmethod
    def can_document_member(
        cls: type[Documenter],
        member: Any,
        membername: str,
        isattr: bool,
        parent: Any,
    ) -> bool: ...
    @override
    def format_args(self) -> str: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def document_members(self, all_members: bool = False) -> None: ...
    @override
    def check_module(self) -> bool: ...

class TaskDirective(PyFunction):
    @override
    def get_signature_prefix(self, sig: str) -> list[nodes.Node]: ...

def autodoc_skip_member_handler(
    app: Any,
    what: object,
    name: object,
    obj: BaseTask[Any, Any],
    skip: bool,
    options: object,
) -> Literal[False] | None: ...
def setup(app: Any) -> dict[Literal["parallel_read_safe"], Literal[True]]: ...



from queue import Empty as Empty

__all__ = (
    "AbstractChannel",
    "Base64",
    "BrokerState",
    "Channel",
    "Empty",
    "Management",
    "Message",
    "NotEquivalentError",
    "QoS",
    "Transport",
    "UndeliverableWarning",
    "binding_key_t",
    "queue_binding_t",
)

from kombu.transport.virtual.base import (
    AbstractChannel as AbstractChannel,
)
from kombu.transport.virtual.base import (
    Base64 as Base64,
)
from kombu.transport.virtual.base import (
    BrokerState as BrokerState,
)
from kombu.transport.virtual.base import (
    Channel as Channel,
)
from kombu.transport.virtual.base import (
    Management as Management,
)
from kombu.transport.virtual.base import (
    Message as Message,
)
from kombu.transport.virtual.base import (
    NotEquivalentError as NotEquivalentError,
)
from kombu.transport.virtual.base import (
    QoS as QoS,
)
from kombu.transport.virtual.base import (
    Transport as Transport,
)
from kombu.transport.virtual.base import (
    UndeliverableWarning as UndeliverableWarning,
)
from kombu.transport.virtual.base import (
    binding_key_t as binding_key_t,
)
from kombu.transport.virtual.base import (
    queue_binding_t as queue_binding_t,
)



from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport

DEFAULT_PORT: int
KZ_CONNECTION_ERRORS: tuple[type[Exception], ...]
KZ_CHANNEL_ERRORS: tuple[type[Exception], ...]

class Channel(VirtualChannel):
    def __init__(self, connection: Any, **kwargs: Any) -> None: ...
    @property
    def client(self) -> Any: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    channel_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]



from logging import Logger

import click
from celery.app.base import Celery
from celery.bin.base import CeleryDaemonCommand
from typing_extensions import override

class Autoscale(click.ParamType):
    name: str
    @override
    def convert(
        self,
        value: str | None,
        param: click.Parameter | None,
        ctx: click.Context | None,
    ) -> tuple[int, int] | None: ...

class CeleryBeat(click.ParamType):
    name: str
    @override
    def convert(
        self,
        value: str | None,
        param: click.Parameter | None,
        ctx: click.Context | None,
    ) -> bool: ...

class Hostname(click.types.StringParamType):
    name: str

class WorkersPool(click.Choice):
    name: str
    def __init__(self) -> None: ...

AUTOSCALE: Autoscale
CELERY_BEAT: CeleryBeat
C_FAKEFORK: str | None
HOSTNAME: Hostname
WORKERS_POOL: WorkersPool
logger: Logger

def detach(
    path: str,
    argv: list[str],
    logfile: str | None = None,
    pidfile: str | None = None,
    uid: int | None = None,
    gid: int | None = None,
    umask: int | None = None,
    workdir: str | None = None,
    fake: bool = False,
    app: Celery | None = None,
    executable: str | None = None,
    hostname: str | None = None,
) -> int: ...

worker: CeleryDaemonCommand



from typing import Any

__all__ = ("Beat",)

from celery.app.base import Celery
from celery.beat import Service as BeatService

class Beat:
    Service: type[BeatService]
    app: Celery | None

    def __init__(
        self,
        max_interval: int | None = None,
        app: Celery | None = None,
        socket_timeout: int = 30,
        pidfile: str | None = None,
        no_color: bool | None = None,
        loglevel: str = "WARN",
        logfile: str | None = None,
        schedule: str | None = None,
        scheduler: str | None = None,
        scheduler_cls: str | None = None,
        redirect_stdouts: bool | None = None,
        redirect_stdouts_level: str | None = None,
        quiet: bool = False,
        **kwargs: Any,
    ) -> None: ...
    def banner(self, service: BeatService) -> str: ...
    def init_loader(self) -> None: ...
    def install_sync_handler(self, service: BeatService) -> None: ...
    def run(self) -> None: ...
    def set_process_title(self) -> None: ...
    def setup_logging(self, colorize: bool | None = None) -> None: ...
    def start_scheduler(self) -> BeatService: ...
    def startup_info(self, service: BeatService) -> str: ...



from logging import Logger

from kombu.connection import Connection
from kombu.entity import Queue

logger: Logger
MAX_NUMBER_OF_BITS_TO_USE: int
MAX_LEVEL: int
CELERY_DELAYED_DELIVERY_EXCHANGE: str

def level_name(level: int) -> str: ...
def declare_native_delayed_delivery_exchanges_and_queues(
    connection: Connection, queue_type: str
) -> None: ...
def bind_queue_to_native_delayed_delivery_exchange(
    connection: Connection, queue: Queue
) -> None: ...
def calculate_routing_key(countdown: int, routing_key: str) -> str: ...



from queue import Empty as Empty
from queue import Full as Full
from typing import Any, Generic, TypeVar

__all__ = ["JoinableQueue", "Queue", "SimpleQueue"]

_T = TypeVar("_T")

class Queue(Generic[_T]):
    def __init__(self, maxsize: int = ..., *args: Any, **kwargs: Any) -> None: ...
    def get(self, block: bool = ..., timeout: float | None = ...) -> _T: ...
    def put(self, obj: _T, block: bool = ..., timeout: float | None = ...) -> None: ...
    def get_nowait(self) -> _T: ...
    def put_nowait(self, obj: _T) -> None: ...
    def empty(self) -> bool: ...
    def full(self) -> bool: ...
    def qsize(self) -> int: ...
    def close(self) -> None: ...
    def join_thread(self) -> None: ...
    def cancel_join_thread(self) -> None: ...

class JoinableQueue(Queue[_T]):
    def __init__(self, maxsize: int = ..., *args: Any, **kwargs: Any) -> None: ...
    def task_done(self) -> None: ...
    def join(self) -> None: ...

class SimpleQueue(Generic[_T]):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def get(self) -> _T: ...
    def put(self, obj: _T) -> None: ...
    def empty(self) -> bool: ...
    def close(self) -> None: ...
    def get_payload(self) -> Any: ...
    def send_payload(self, value: Any) -> None: ...



from collections.abc import Callable
from typing import Any

from vine.promises import promise

def maybe_promise(p: Any) -> promise | None: ...
def ensure_promise(p: Any) -> promise: ...
def ppartial(p: promise, *args: Any, **kwargs: Any) -> promise: ...
def preplace(p: promise, *args: Any, **kwargs: Any) -> promise: ...
def ready_promise(callback: Callable[..., Any] | None = ..., *args: Any) -> promise: ...
def starpromise(fun: Callable[..., Any], *args: Any, **kwargs: Any) -> promise: ...
def transform(
    filter_: Callable[..., Any],
    callback: Callable[..., Any],
    *filter_args: Any,
    **filter_kwargs: Any,
) -> promise: ...
def wrap(p: Any) -> promise: ...

__all__ = [
    "ensure_promise",
    "maybe_promise",
    "ppartial",
    "preplace",
    "ready_promise",
    "starpromise",
    "transform",
    "wrap",
]



__all__ = ["Array", "RawArray", "RawValue", "Value", "copy", "synchronized"]

from multiprocessing.sharedctypes import Array as Array
from multiprocessing.sharedctypes import RawArray as RawArray
from multiprocessing.sharedctypes import RawValue as RawValue
from multiprocessing.sharedctypes import Value as Value
from multiprocessing.sharedctypes import copy as copy
from multiprocessing.sharedctypes import synchronized as synchronized



import logging
from collections.abc import Callable
from typing import Any

__all__ = ("default",)

logger: logging.Logger

def default(
    task: Any,
    app: Any,
    consumer: Any,
    info: Callable[..., None] = ...,
    error: Callable[..., None] = ...,
    task_reserved: Callable[..., Any] = ...,
    to_system_tz: Callable[..., Any] = ...,
    bytes: type[bytes] = ...,
    proto1_to_proto2: Callable[..., Any] = ...,
) -> Callable[..., Any]: ...



from types import TracebackType
from typing import Any, TypeAlias

__all__ = ["ExceptionInfo", "Traceback"]

class _Code:
    def __init__(self, code: Any) -> None: ...
    @property
    def co_positions(self) -> Any: ...

class _Frame:
    Code = _Code

    def __init__(self, frame: Any) -> None: ...
    @property
    def co_positions(self) -> Any: ...

class Traceback:
    Frame = _Frame
    Code = _Code

    def __init__(
        self, tb: TracebackType | None, max_frames: int = ..., depth: int = ...
    ) -> None: ...

_ExcInfo: TypeAlias = tuple[type[BaseException], BaseException, TracebackType | None]

class ExceptionInfo:
    exception: BaseException | None
    internal: bool
    tb: TracebackType | None
    traceback: str | None

    def __init__(
        self,
        exc_info: _ExcInfo | None = ...,
        internal: bool = ...,
    ) -> None: ...
    @property
    def type(self) -> type[BaseException] | None: ...  # ty: ignore[invalid-type-form]
    @property
    def exc_info(self) -> _ExcInfo | None: ...



from typing import Any

from azure.storage.queue import QueueServiceClient
from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from typing_extensions import override

CHARS_REPLACE_TABLE: dict[int, int]

class Channel(VirtualChannel):
    domain_format: str
    _queue_service: QueueServiceClient | None
    _queue_name_cache: dict[Any, Any]
    no_ack: bool
    _noack_queues: set[Any]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def basic_consume(
        self, queue: str, no_ack: bool, *args: Any, **kwargs: Any
    ) -> str: ...
    def entity_name(self, name: str, table: dict[int, int] = ...) -> str: ...
    @property
    def queue_service(self) -> QueueServiceClient: ...
    @property
    def conninfo(self) -> Any: ...
    @property
    def transport_options(self) -> dict[str, Any]: ...
    @cached_property
    def queue_name_prefix(self) -> str: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

    polling_interval: int  # pyright: ignore[reportIncompatibleVariableOverride]
    default_port: int | None
    can_parse_url: bool

    @staticmethod
    def parse_uri(uri: str) -> tuple[Any, str]: ...
    @classmethod
    @override
    def as_uri(cls, uri: str, include_password: bool = ..., mask: str = ...) -> str: ...  # pyright: ignore[reportIncompatibleMethodOverride]



from collections.abc import Callable
from typing import Any, ClassVar

from celery.bootsteps import ConsumerStep
from typing_extensions import override

__all__ = ("Gossip",)

class Gossip(ConsumerStep):
    name: ClassVar[str]  # pyright: ignore[reportIncompatibleVariableOverride]
    compatible_transports: set[str]

    def __init__(
        self,
        c: Any,
        without_gossip: bool = False,
        interval: float = 5.0,
        heartbeat_interval: float = 2.0,
        **kwargs: Any,
    ) -> None: ...
    @override
    def create(self, parent: Any) -> Any: ...
    @override
    def start(self, c: Any) -> None: ...  # ty: ignore[invalid-method-override]
    @override
    def stop(self, c: Any) -> None: ...  # ty: ignore[invalid-method-override]
    @override
    def shutdown(self, c: Any) -> None: ...
    @override
    def include_if(self, parent: Any) -> bool: ...
    @override
    def info(self, obj: Any) -> dict[str, Any]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def call_task(self, task: Any) -> Any: ...
    def compatible_transport(self, app: Any) -> bool: ...
    def election(
        self, id: str, topic: str, action: Callable[..., Any] | None = None
    ) -> None: ...
    @override
    def get_consumers(self, channel: Any) -> list[Any]: ...
    def on_elect(self, event: Any) -> None: ...
    def on_elect_ack(self, event: Any) -> None: ...
    def on_message(self, prepare: Any, message: Any) -> None: ...
    def on_node_join(self, worker: Any) -> None: ...
    def on_node_leave(self, worker: Any) -> None: ...
    def on_node_lost(self, worker: Any) -> None: ...
    def periodic(self) -> None: ...
    def register_timer(self) -> None: ...



from collections.abc import Callable, Iterator
from contextlib import contextmanager
from signal import Handlers
from signal import Signals as SignalEnum
from typing import Any

from typing_extensions import Self

__all__ = (
    "EX_FAILURE",
    "EX_OK",
    "EX_UNAVAILABLE",
    "EX_USAGE",
    "IS_WINDOWS",
    "SIGMAP",
    "SYSTEM",
    "DaemonContext",
    "IS_macOS",
    "LockFailed",
    "Pidfile",
    "close_open_fds",
    "create_pidlock",
    "detached",
    "fd_by_path",
    "get_errno_name",
    "get_fdmax",
    "ignore_errno",
    "initgroups",
    "isatty",
    "maybe_drop_privileges",
    "parse_gid",
    "parse_uid",
    "pyimplementation",
    "set_mp_process_title",
    "set_process_title",
    "setgid",
    "setgroups",
    "setuid",
    "signal_name",
    "signals",
)

DAEMON_WORKDIR: str
EX_OK: int
EX_FAILURE: int
EX_UNAVAILABLE: int
EX_USAGE: int
SYSTEM: str
IS_macOS: bool
IS_WINDOWS: bool
SIGMAP: dict[int, str]

class Signals:
    default: Handlers
    ignored: Handlers
    def arm_alarm(self, seconds: float) -> None: ...
    def ignore(self, *signals: int) -> None: ...
    def reset(self, sig: int) -> None: ...
    def reset_alarm(self) -> None: ...
    def signum(self, name: int | str) -> int: ...
    def supported(self, name: int | str) -> bool: ...
    def update(
        self,
        _d_: dict[int | SignalEnum, Callable[[int, Any], None]] | None = None,
        **sigmap: Callable[[int, Any], None],
    ) -> None: ...
    def __getitem__(self, name: int | str) -> Callable[[int, Any], None] | None: ...
    def __setitem__(
        self, name: int | str, handler: Callable[[int, Any], None]
    ) -> None: ...

signals: Signals

def pyimplementation() -> str: ...
def signal_name(signum: int) -> str | None: ...
def get_errno_name(n: int) -> str: ...
def get_fdmax(default: int | None = None) -> int: ...
def close_open_fds(keep: set[int] | None = None) -> None: ...
def fd_by_path(paths: list[str]) -> list[int]: ...
@contextmanager
def ignore_errno(*errnos: int) -> Iterator[None]: ...
def set_process_title(progname: str, info: str | None = None) -> None: ...
def set_mp_process_title(
    progname: str, info: str | None = None, hostname: str | None = None
) -> None: ...
def maybe_drop_privileges(
    uid: int | str | None = None, gid: int | str | None = None
) -> None: ...
def setgroups(groups: list[int]) -> None: ...
def initgroups(uid: int, gid: int) -> None: ...
def setgid(gid: int | str) -> None: ...
def setuid(uid: int | str) -> None: ...
def parse_uid(uid: int | str) -> int: ...
def parse_gid(gid: int | str) -> int: ...
def isatty(fh: Any) -> bool: ...
def detached(
    logfile: str | None = None,
    pidfile: str | None = None,
    uid: int | str | None = None,
    gid: int | str | None = None,
    umask: int = 0,
    workdir: str | None = None,
    fake: bool = False,
    **opts: Any,
) -> Any: ...
def create_pidlock(pidfile: str) -> Any: ...

class DaemonContext:
    open_fds: set[int] | None
    pidfile: str | None
    workdir: str
    umask: int | None

    def __init__(
        self,
        pidfile: str | None = None,
        workdir: str | None = None,
        umask: int | None = None,
        fake: bool = False,
        after_chdir: Callable[[], None] | None = None,
        after_forkers: bool = True,
        **kwargs: Any,
    ) -> None: ...
    def open(self) -> DaemonContext: ...
    def close(self, *args: Any) -> None: ...
    def redirect_to_null(self, fd: int) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: object) -> None: ...

class Pidfile:
    path: str | None
    def __init__(self, path: str) -> None: ...
    def acquire(self) -> Pidfile: ...
    def is_locked(self) -> bool: ...
    def release(self, *args: Any) -> None: ...
    def read_pid(self) -> int | None: ...
    def remove(self) -> None: ...
    def remove_if_stale(self) -> bool: ...
    def write_pid(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: object) -> None: ...

class LockFailed(Exception): ...



import threading
from collections.abc import Callable, Iterable, Mapping
from queue import Queue as Queue
from threading import BoundedSemaphore as BoundedSemaphore

# Note: Condition is a custom class in billiard.dummy, not threading.Condition
from threading import Event as Event
from threading import Lock as Lock
from threading import Semaphore as Semaphore
from typing import Any

from billiard.connection import Pipe as Pipe
from typing_extensions import override

__all__ = [
    "BoundedSemaphore",
    "Condition",
    "Event",
    "JoinableQueue",
    "Lock",
    "Manager",
    "Pipe",
    "Pool",
    "Process",
    "Queue",
    "RLock",
    "Semaphore",
    "active_children",
    "current_process",
    "freeze_support",
]

class DummyProcess(threading.Thread):
    def __init__(
        self,
        group: None = ...,
        target: Callable[..., object] | None = ...,
        name: str | None = ...,
        args: Iterable[Any] = ...,
        kwargs: Mapping[str, Any] = ...,
    ) -> None: ...
    @override
    def start(self) -> None: ...
    @property
    def exitcode(self) -> int | None: ...

Process = DummyProcess
current_process = threading.current_thread

def freeze_support() -> None: ...
def active_children() -> list[DummyProcess]: ...
def Manager() -> Any: ...
def Pool(
    processes: int | None = ...,
    initializer: Callable[..., object] | None = ...,
    initargs: Iterable[Any] = ...,
) -> Any: ...
def RLock(*args: Any, **kwargs: Any) -> threading.RLock: ...

class Condition:
    def __init__(self, lock: Any = ...) -> None: ...
    def __enter__(self) -> bool: ...
    def __exit__(self, *args: object) -> None: ...
    def wait(self, timeout: float | None = ...) -> bool: ...
    def wait_for(
        self, predicate: Callable[[], bool], timeout: float | None = ...
    ) -> bool: ...
    def notify(self, n: int = ...) -> None: ...
    def notify_all(self) -> None: ...

JoinableQueue = Queue



from typing import Any

__all__ = ("Autoscaler", "WorkerComponent")

from celery.bootsteps import StartStopStep
from celery.concurrency.base import BasePool
from celery.utils.threads import bgThread
from typing_extensions import override

class WorkerComponent(StartStopStep):
    enabled: Any
    def __init__(self, w: Any, **kwargs: Any) -> None: ...
    @override
    def create(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self, w: Any
    ) -> Any: ...
    def register_with_event_loop(self, w: Any, hub: Any) -> None: ...
    @override
    def info(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self, w: Any
    ) -> Any: ...

class Autoscaler(bgThread):
    pool: BasePool
    mutex: Any
    max_concurrency: int
    min_concurrency: int
    keepalive: int | float
    worker: Any
    def __init__(
        self,
        pool: BasePool,
        max_concurrency: int,
        min_concurrency: int = ...,
        worker: Any | None = ...,
        keepalive: float = ...,
        mutex: Any | None = ...,
    ) -> None: ...
    @override
    def body(self) -> None: ...
    def maybe_scale(self, req: Any | None = ...) -> None: ...
    def update(self, max: Any | None = ..., min: Any | None = ...) -> Any: ...
    def scale_up(self, n: Any) -> Any: ...
    def scale_down(self, n: Any) -> Any: ...
    def info(self) -> Any: ...
    @property
    def qty(self) -> Any: ...
    @property
    def processes(self) -> Any: ...



from typing import Any

from celery import bootsteps
from typing_extensions import override

__all__ = ("Beat", "Consumer", "Hub", "Pool", "StateDB", "Timer")

class Timer(bootsteps.Step):
    def __init__(self, parent: Any, **kwargs: Any) -> None: ...
    @override
    def create(self, w: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    def on_timer_error(self, exc: Exception) -> None: ...
    def on_timer_tick(self, delay: float) -> None: ...

class Hub(bootsteps.StartStopStep):
    def __init__(self, w: Any, **kwargs: Any) -> None: ...
    @override
    def create(self, w: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def include_if(self, w: Any) -> bool: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def start(self, w: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def stop(self, w: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def terminate(self, w: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]

class Pool(bootsteps.StartStopStep):
    def __init__(
        self, w: Any, autoscale: tuple[int, int] | None = None, **kwargs: Any
    ) -> None: ...
    @override
    def close(self, w: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def create(self, w: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def info(self, w: Any) -> dict[str, Any]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def register_with_event_loop(self, w: Any, hub: Any) -> None: ...
    @override
    def terminate(self, w: Any) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]

class Beat(bootsteps.StartStopStep):
    def __init__(self, w: Any, beat: bool = False, **kwargs: Any) -> None: ...
    @override
    def create(self, w: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]

class StateDB(bootsteps.Step):
    def __init__(self, w: Any, **kwargs: Any) -> None: ...
    @override
    def create(self, w: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]

class Consumer(bootsteps.StartStopStep):
    def __init__(self, parent: Any, **kwargs: Any) -> None: ...
    @override
    def create(self, w: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]



from typing import Any, TextIO

__all__ = ("Dumper", "evdump")

class Dumper:
    out: TextIO

    def __init__(self, out: TextIO = ...) -> None: ...
    def say(self, msg: str) -> None: ...
    def format_task_event(
        self,
        hostname: str,
        timestamp: float,
        type: str,
        task: str,
        event: dict[str, Any],
    ) -> str: ...
    def on_event(self, ev: dict[str, Any]) -> None: ...

def evdump(app: Any | None = None, out: TextIO = ...) -> None: ...



from typing import Any

from typing_extensions import override

class SASL:
    @property
    def mechanism(self) -> bytes | None: ...
    def start(self, connection: Any) -> bytes | None: ...

class PLAIN(SASL):
    mechanism: bytes  # pyright: ignore[reportIncompatibleMethodOverride]
    username: str
    password: str

    def __init__(self, username: str, password: str) -> None: ...
    @override
    def start(self, connection: Any) -> bytes: ...

class AMQPLAIN(SASL):
    mechanism: bytes  # pyright: ignore[reportIncompatibleMethodOverride]
    username: str
    password: str

    def __init__(self, username: str, password: str) -> None: ...
    @override
    def start(self, connection: Any) -> bytes: ...

class EXTERNAL(SASL):
    mechanism: bytes  # pyright: ignore[reportIncompatibleMethodOverride]

    @override
    def start(self, connection: Any) -> bytes: ...

class RAW(SASL):
    mechanism: bytes | None  # pyright: ignore[reportIncompatibleMethodOverride]
    response: bytes | None

    def __init__(self, mechanism: bytes | None, response: bytes | None) -> None: ...
    @override
    def start(self, connection: Any) -> bytes | None: ...

class GSSAPI(SASL):
    mechanism: bytes | None  # pyright: ignore[reportIncompatibleMethodOverride]
    client_name: str | None
    service: bytes
    rdns: bool
    fail_soft: bool

    def __init__(
        self,
        client_name: str | None = ...,
        service: bytes = ...,
        rdns: bool = ...,
        fail_soft: bool = ...,
    ) -> None: ...
    @override
    def start(self) -> bytes | None: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]



from logging import Logger
from typing import Any

from confluent_kafka import (  # type: ignore[import-untyped]  # pyright: ignore[reportMissingTypeStubs]
    KafkaException,  # pyright: ignore[reportUnknownVariableType]
)
from confluent_kafka.admin import (  # type: ignore[import-untyped]  # pyright: ignore[reportMissingTypeStubs]
    AdminClient,
)
from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Message as VirtualMessage
from kombu.transport.virtual import QoS as VirtualQoS
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from typing_extensions import override

logger: Logger
DEFAULT_PORT: int
KAFKA_CONNECTION_ERRORS: tuple[type[Exception], ...]
KAFKA_CHANNEL_ERRORS: tuple[type[Exception], ...]

class NoBrokersAvailable(KafkaException):  # pyright: ignore[reportUntypedBaseClass]
    retriable: bool

class Message(VirtualMessage):
    topic: str | None

    def __init__(
        self, payload: dict[str, Any], channel: Any | None = ..., **kwargs: Any
    ) -> None: ...

_Message = Message

class QoS(VirtualQoS):
    _not_yet_acked: dict[str, Any]

    @override
    def can_consume(self) -> bool: ...
    @override
    def can_consume_max_estimate(self) -> int: ...
    @override
    def append(self, message: Any, delivery_tag: str) -> None: ...
    @override
    def get(self, delivery_tag: str) -> Any: ...
    @override
    def ack(self, delivery_tag: str) -> None: ...
    @override
    def reject(self, delivery_tag: str, requeue: bool = ...) -> None: ...
    @override
    def restore_unacked_once(self, stderr: Any | None = ...) -> None: ...

_QoS = QoS

class Channel(VirtualChannel):
    QoS: type[_QoS]  # pyright: ignore[reportIncompatibleVariableOverride]
    Message: type[_Message]  # pyright: ignore[reportIncompatibleVariableOverride]

    default_wait_time_seconds: int
    default_connection_wait_time_seconds: int

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def sanitize_queue_name(self, queue: str) -> str: ...
    @override
    def close(self) -> None: ...
    @property
    def client(self) -> AdminClient: ...
    @property
    def options(self) -> dict[str, Any]: ...
    @property
    def conninfo(self) -> Any: ...
    @cached_property
    def wait_time_seconds(self) -> int: ...
    @cached_property
    def connection_wait_time_seconds(self) -> int: ...
    @cached_property
    def common_config(self) -> dict[str, Any]: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

    default_port: int  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    recoverable_connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]

    def __init__(self, client: Any, **kwargs: Any) -> None: ...
    @override
    def as_uri(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, uri: str, include_password: bool = ..., mask: str = ...
    ) -> None: ...
    @override
    def driver_version(self) -> str: ...
    @override
    def establish_connection(self) -> Any: ...
    @override
    def close_connection(self, connection: Any) -> None: ...



__all__ = ["BufferWrapper"]

class BufferWrapper:
    def __init__(self, size: int) -> None: ...
    def create_memoryview(self) -> memoryview: ...
    def get_address(self) -> int: ...
    def get_size(self) -> int: ...



from typing import Any, TypeAlias

from kombu.connection import Connection, ConnectionPool
from kombu.messaging import Producer
from typing_extensions import override

__all__ = (
    "PoolGroup",
    "ProducerPool",
    "connections",
    "get_limit",
    "producers",
    "register_group",
    "reset",
    "set_limit",
)

_ProducerType: TypeAlias = Producer

class PoolGroup(dict[Any, Any]):
    def __init__(
        self, limit: int | None = ..., close_after_fork: bool = ...
    ) -> None: ...
    def __missing__(self, resource: Any) -> Any: ...
    def create(self, resource: Any, limit: int | None) -> Any: ...

class Connections(PoolGroup):
    @override
    def create(self, connection: Connection, limit: int | None) -> ConnectionPool: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]

class Producers(PoolGroup):
    @override
    def create(self, connection: Connection, limit: int | None) -> ProducerPool: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]

class ProducerPool:
    Producer: type[_ProducerType]
    connections: ConnectionPool
    limit: int | None
    close_after_fork: bool

    def __init__(
        self,
        connections: ConnectionPool,
        Producer: type[_ProducerType] = ...,
        limit: int | None = ...,
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    def setup(self) -> None: ...
    def create_producer(self) -> _ProducerType: ...
    def new(self) -> _ProducerType: ...
    def acquire(
        self, block: bool = ..., timeout: float | None = ...
    ) -> _ProducerType: ...
    def prepare(self, p: _ProducerType) -> _ProducerType: ...
    def release(self, resource: _ProducerType) -> None: ...
    def close_resource(self, resource: _ProducerType) -> None: ...
    def _acquire_connection(self) -> Connection: ...

def _all_pools() -> list[PoolGroup]: ...

connections: Connections
producers: Producers

def register_group(group: PoolGroup) -> PoolGroup: ...
def get_limit() -> int | None: ...
def set_limit(
    limit: int,
    force: bool = ...,
    reset_after: bool = ...,
    ignore_errors: bool = ...,
) -> int | None: ...
def reset(*args: Any, **kwargs: Any) -> None: ...



from abc import ABC, abstractmethod
from collections.abc import Callable
from types import TracebackType
from typing import Any

class Thenable(ABC):
    @abstractmethod
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    @abstractmethod
    def cancel(self) -> None: ...
    @classmethod
    def register(cls, other: type[Any]) -> type[Any]: ...
    @abstractmethod
    def then(
        self,
        on_success: Callable[..., Any],
        on_error: Callable[..., Any] | None = ...,
    ) -> Thenable: ...
    @abstractmethod
    def throw(
        self,
        exc: BaseException | None = ...,
        tb: TracebackType | None = ...,
        propagate: bool = ...,
    ) -> None: ...

__all__ = ["Thenable"]



from collections.abc import Callable, Generator, Sequence
from functools import partial
from typing import Any, TextIO, TypeVar

from celery.app.base import Celery
from celery.result import AsyncResult

_T = TypeVar("_T")

E_STILL_WAITING: str

humanize_seconds: partial[str]

class Sentinel(Exception): ...

class ManagerMixin:
    def _init_manager(
        self,
        block_timeout: float = 1800.0,
        no_join: bool = False,
        stdout: TextIO | None = None,
        stderr: TextIO | None = None,
    ) -> None: ...
    def remark(self, s: str, sep: str = "-") -> None: ...
    def missing_results(self, r: Sequence[AsyncResult[Any]]) -> Sequence[str]: ...
    def wait_for(
        self,
        fun: Callable[..., _T],
        catch: Sequence[Any],
        desc: str = "thing",
        args: tuple[Any, ...] = (),
        kwargs: dict[str, Any] | None = None,
        errback: Callable[..., Any] | None = None,
        max_retries: int = 10,
        interval_start: float = 0.1,
        interval_step: float = 0.5,
        interval_max: float = 5.0,
        emit_warning: bool = False,
        **options: Any,
    ) -> _T: ...
    def ensure_not_for_a_while(
        self,
        fun: Callable[..., _T],
        catch: Sequence[Any],
        desc: str = "thing",
        max_retries: int = 20,
        interval_start: float = 0.1,
        interval_step: float = 0.02,
        interval_max: float = 1.0,
        emit_warning: bool = False,
        **options: Any,
    ) -> _T: ...
    def retry_over_time(self, *args: Any, **kwargs: Any) -> Any: ...
    def join(
        self,
        r: Any,
        propagate: bool = False,
        max_retries: int = 10,
        **kwargs: Any,
    ) -> Any: ...
    def inspect(self, timeout: float = 3.0) -> Any: ...
    def query_tasks(
        self, ids: Any, timeout: float = 0.5
    ) -> Generator[tuple[str, dict[str, tuple[str, Any]]], None, None]: ...
    def query_task_states(
        self, ids: list[str], timeout: float = 0.5
    ) -> dict[str, set[str]]: ...
    def assert_accepted(
        self,
        ids: list[str],
        interval: float = 0.5,
        desc: str = "waiting for tasks to be accepted",
        **policy: Any,
    ) -> bool | Sentinel: ...
    def assert_received(
        self,
        ids: list[str],
        interval: float = 0.5,
        desc: str = "waiting for tasks to be received",
        **policy: Any,
    ) -> bool | Sentinel: ...
    def assert_result_tasks_in_progress_or_completed(
        self,
        async_results: Sequence[AsyncResult[Any]],
        interval: float = 0.5,
        desc: str = "waiting for tasks to be started or completed",
        **policy: Any,
    ) -> bool | Sentinel: ...
    def assert_task_state_from_result(
        self,
        fun: Callable[..., _T],
        results: Sequence[AsyncResult[Any]],
        interval: float = 0.5,
        **policy: Any,
    ) -> _T | Sentinel: ...
    @staticmethod
    def is_result_task_in_progress(
        results: Sequence[AsyncResult[Any]], **kwargs: Any
    ) -> bool: ...
    def assert_task_worker_state(
        self,
        fun: Callable[..., _T],
        ids: list[str],
        interval: float = 0.5,
        **policy: Any,
    ) -> _T | Sentinel: ...
    def is_received(self, ids: list[str], **kwargs: Any) -> bool: ...
    def is_accepted(self, ids: list[str], **kwargs: Any) -> bool: ...
    def _ids_matches_state(
        self, expected_states: list[str], ids: list[str], timeout: float = 0.5
    ) -> bool: ...
    def true_or_raise(
        self, fun: Callable[..., _T], *args: Any, **kwargs: Any
    ) -> _T | Sentinel: ...
    def wait_until_idle(self) -> None: ...

class Manager(ManagerMixin):
    def __init__(self, app: Celery, **kwargs: Any) -> None: ...



from typing import Any

from celery.backends.base import Backend

__all__ = ("CassandraBackend",)

class CassandraBackend(Backend):
    servers: list[str] | None
    bundle_path: str | None

    def __init__(
        self,
        servers: list[str] | None = None,
        keyspace: str | None = None,
        table: str | None = None,
        entry_ttl: int | None = None,
        port: int | None = None,
        bundle_path: str | None = None,
        **kwargs: Any,
    ) -> None: ...



from typing import Any

from kombu.asynchronous.aws.sqs.connection import AsyncSQSConnection

def connect_sqs(
    aws_access_key_id: str | None = ...,
    aws_secret_access_key: str | None = ...,
    **kwargs: Any,
) -> AsyncSQSConnection: ...



from datetime import datetime
from typing import Any, ClassVar

__all__ = ("Task", "TaskExtended", "TaskSet")

# Note: These are SQLAlchemy declarative models with DeclarativeMeta metaclass
# stubtest cannot reconcile the InstrumentedAttribute class-level types with
# the instance-level types users actually see. Using plain classes here since
# users access these as regular attributes on instances.

class Task:
    __tablename__: ClassVar[str]
    __table_args__: ClassVar[dict[str, Any]]
    __table__: ClassVar[Any]
    __mapper__: ClassVar[Any]

    id: int | None
    task_id: str
    status: str
    result: bytes | None
    date_done: datetime | None
    traceback: str | None

    def __init__(self, task_id: str) -> None: ...
    def to_dict(self) -> dict[str, Any]: ...
    @classmethod
    def configure(cls, schema: str | None = None, name: str | None = None) -> None: ...

class TaskExtended(Task):
    name: str | None
    args: bytes | None
    kwargs: bytes | None
    worker: str | None
    retries: int | None
    queue: str | None
    children: bytes | None

    def __init__(self, task_id: str) -> None: ...

class TaskSet:
    __tablename__: ClassVar[str]
    __table_args__: ClassVar[dict[str, Any]]
    __table__: ClassVar[Any]
    __mapper__: ClassVar[Any]

    id: int | None
    taskset_id: str
    result: bytes | None
    date_done: datetime | None

    def __init__(self, taskset_id: str, result: bytes | None) -> None: ...
    def to_dict(self) -> dict[str, Any]: ...
    @classmethod
    def configure(cls, schema: str | None = None, name: str | None = None) -> None: ...



from collections.abc import Callable, Mapping
from typing import Any, TypeAlias

from amqp.basic_message import Message

__all__ = ("Channel",)

_MessageType: TypeAlias = Message

class Channel:
    Message: type[_MessageType]

    connection: Any
    channel_id: int | None
    auto_decode: bool
    is_open: bool

    def __init__(
        self,
        connection: Any,
        channel_id: int | None = ...,
        auto_decode: bool = ...,
        on_open: Callable[..., Any] | None = ...,
    ) -> None: ...
    def open(self) -> None: ...
    def close(
        self,
        reply_code: int = ...,
        reply_text: str = ...,
        method_sig: tuple[int, int] = ...,
        argsig: str = ...,
    ) -> None: ...
    def collect(self) -> None: ...
    def flow(self, active: bool) -> None: ...
    def exchange_declare(
        self,
        exchange: str,
        type: str,
        passive: bool = ...,
        durable: bool = ...,
        auto_delete: bool = ...,
        nowait: bool = ...,
        arguments: Mapping[str, Any] | None = ...,
        argsig: str = ...,
    ) -> None: ...
    def exchange_delete(
        self,
        exchange: str,
        if_unused: bool = ...,
        nowait: bool = ...,
        argsig: str = ...,
    ) -> None: ...
    def exchange_bind(
        self,
        destination: str,
        source: str = ...,
        routing_key: str = ...,
        nowait: bool = ...,
        arguments: Mapping[str, Any] | None = ...,
        argsig: str = ...,
    ) -> None: ...
    def exchange_unbind(
        self,
        destination: str,
        source: str = ...,
        routing_key: str = ...,
        nowait: bool = ...,
        arguments: Mapping[str, Any] | None = ...,
        argsig: str = ...,
    ) -> None: ...
    def queue_declare(
        self,
        queue: str = ...,
        passive: bool = ...,
        durable: bool = ...,
        exclusive: bool = ...,
        auto_delete: bool = ...,
        nowait: bool = ...,
        arguments: Mapping[str, Any] | None = ...,
        argsig: str = ...,
    ) -> tuple[str, int, int] | None: ...
    def queue_delete(
        self,
        queue: str = ...,
        if_unused: bool = ...,
        if_empty: bool = ...,
        nowait: bool = ...,
        argsig: str = ...,
    ) -> int | None: ...
    def queue_bind(
        self,
        queue: str,
        exchange: str = ...,
        routing_key: str = ...,
        nowait: bool = ...,
        arguments: Mapping[str, Any] | None = ...,
        argsig: str = ...,
    ) -> None: ...
    def queue_unbind(
        self,
        queue: str,
        exchange: str,
        routing_key: str = ...,
        nowait: bool = ...,
        arguments: Mapping[str, Any] | None = ...,
        argsig: str = ...,
    ) -> None: ...
    def queue_purge(
        self, queue: str = ..., nowait: bool = ..., argsig: str = ...
    ) -> int | None: ...
    def basic_qos(
        self,
        prefetch_size: int,
        prefetch_count: int,
        a_global: bool,
        argsig: str = ...,
    ) -> None: ...
    def basic_consume(
        self,
        queue: str = ...,
        consumer_tag: str = ...,
        no_local: bool = ...,
        no_ack: bool = ...,
        exclusive: bool = ...,
        nowait: bool = ...,
        callback: Callable[..., Any] | None = ...,
        arguments: Mapping[str, Any] | None = ...,
        on_cancel: Callable[..., Any] | None = ...,
        argsig: str = ...,
    ) -> str: ...
    def basic_cancel(
        self, consumer_tag: str, nowait: bool = ..., argsig: str = ...
    ) -> None: ...
    def basic_get(
        self, queue: str = ..., no_ack: bool = ..., argsig: str = ...
    ) -> _MessageType | None: ...
    def basic_ack(
        self, delivery_tag: int, multiple: bool = ..., argsig: str = ...
    ) -> None: ...
    def basic_reject(
        self, delivery_tag: int, requeue: bool, argsig: str = ...
    ) -> None: ...
    def basic_recover(self, requeue: bool = ...) -> None: ...
    def basic_recover_async(self, requeue: bool = ...) -> None: ...
    def basic_publish(
        self,
        msg: _MessageType,
        exchange: str = ...,
        routing_key: str = ...,
        mandatory: bool = ...,
        immediate: bool = ...,
        timeout: float | None = ...,
        confirm_timeout: float | None = ...,
        argsig: str = ...,
    ) -> None: ...
    def basic_publish_confirm(self, *args: Any, **kwargs: Any) -> None: ...
    def tx_select(self) -> None: ...
    def tx_commit(self) -> None: ...
    def tx_rollback(self) -> None: ...
    def confirm_select(self, nowait: bool = ...) -> None: ...
    def dispatch_method(
        self, method_sig: tuple[int, int], payload: bytes, content: Any
    ) -> None: ...
    def send_method(
        self,
        sig: tuple[int, int],
        format: str | None = ...,
        args: tuple[Any, ...] | None = ...,
        content: Any | None = ...,
        wait: Any | None = ...,
        callback: Callable[..., Any] | None = ...,
        returns_tuple: bool = ...,
    ) -> Any: ...
    def wait(
        self,
        method: Any,
        callback: Callable[..., Any] | None = ...,
        timeout: float | None = ...,
        returns_tuple: bool = ...,
    ) -> Any: ...
    def then(
        self,
        on_success: Callable[..., Any],
        on_error: Callable[..., Any] | None = ...,
    ) -> Any: ...



from collections.abc import Callable, Mapping
from socket import socket
from typing import Any, TypeAlias

from amqp.channel import Channel as _ChannelType
from typing_extensions import Self

__all__ = ("Connection",)

_ChannelTypeAlias: TypeAlias = _ChannelType

class Connection:
    Channel: type[_ChannelTypeAlias]

    host: str
    userid: str
    password: str
    login_method: str | None
    virtual_host: str
    locale: str
    client_properties: dict[str, Any]
    ssl: bool | dict[str, Any]
    connect_timeout: float | None
    channel_max: int | None
    frame_max: int
    heartbeat: int | None
    confirm_publish: bool
    read_timeout: float | None
    write_timeout: float | None

    bytes_sent: int
    bytes_recv: int
    last_heartbeat_sent: int
    last_heartbeat_received: int
    channel_errors: tuple[type[Exception], ...]
    connection_errors: tuple[type[Exception], ...]
    recoverable_channel_errors: tuple[type[Exception], ...]
    recoverable_connection_errors: tuple[type[Exception], ...]

    client_heartbeat: int | None
    server_heartbeat: int | None
    prev_recv: float | None
    prev_sent: float | None
    library_properties: dict[str, Any]
    negotiate_capabilities: dict[str, bool]

    auto_decode: bool
    channel_id: int | None
    connection: Any
    is_closing: bool
    method_queue: list[Any]

    def __init__(
        self,
        host: str = ...,
        userid: str = ...,
        password: str = ...,
        login_method: str | None = ...,
        login_response: Any | None = ...,
        authentication: tuple[Any, ...] = ...,
        virtual_host: str = ...,
        locale: str = ...,
        client_properties: dict[str, Any] | None = ...,
        ssl: bool | dict[str, Any] = ...,
        connect_timeout: float | None = ...,
        channel_max: int | None = ...,
        frame_max: int | None = ...,
        heartbeat: int = ...,
        on_open: Callable[..., Any] | None = ...,
        on_blocked: Callable[..., Any] | None = ...,
        on_unblocked: Callable[..., Any] | None = ...,
        confirm_publish: bool = ...,
        on_tune_ok: Callable[..., Any] | None = ...,
        read_timeout: float | None = ...,
        write_timeout: float | None = ...,
        socket_settings: Mapping[str, Any] | None = ...,
        frame_handler: Callable[..., Any] = ...,
        frame_writer: Callable[..., Any] = ...,
        **kwargs: Any,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *eargs: object) -> None: ...
    def Transport(
        self,
        host: str,
        connect_timeout: float | None,
        ssl: bool | dict[str, Any] = ...,
        read_timeout: float | None = ...,
        write_timeout: float | None = ...,
        socket_settings: Mapping[str, Any] | None = ...,
        **kwargs: Any,
    ) -> Any: ...
    def connect(self, callback: Callable[..., Any] | None = ...) -> None: ...
    def close(
        self,
        reply_code: int = ...,
        reply_text: str = ...,
        method_sig: tuple[int, int] = ...,
        argsig: str = ...,
    ) -> None: ...
    def channel(
        self, channel_id: int | None = ..., callback: Callable[..., Any] | None = ...
    ) -> _ChannelTypeAlias: ...
    def drain_events(self, timeout: float | None = ...) -> None: ...
    def blocking_read(self, timeout: float | None = ...) -> None: ...
    def collect(self) -> None: ...
    def heartbeat_tick(self, rate: int = ...) -> None: ...
    def send_heartbeat(self) -> None: ...
    def is_alive(self) -> bool: ...
    def dispatch_method(
        self, method_sig: tuple[int, int], payload: bytes, content: Any
    ) -> None: ...
    def on_inbound_method(
        self,
        channel_id: int,
        method_sig: tuple[int, int],
        payload: bytes,
        content: Any,
    ) -> None: ...
    def send_method(
        self,
        sig: tuple[int, int],
        format: str | None = ...,
        args: tuple[Any, ...] | None = ...,
        content: Any | None = ...,
        wait: Any | None = ...,
        callback: Callable[..., Any] | None = ...,
        returns_tuple: bool = ...,
    ) -> Any: ...
    def wait(
        self,
        method: Any,
        callback: Callable[..., Any] | None = ...,
        timeout: float | None = ...,
        returns_tuple: bool = ...,
    ) -> Any: ...
    def then(
        self,
        on_success: Callable[..., Any],
        on_error: Callable[..., Any] | None = ...,
    ) -> Any: ...
    @property
    def connected(self) -> bool: ...
    @property
    def sock(self) -> socket | None: ...
    @property
    def server_capabilities(self) -> dict[str, Any]: ...
    @property
    def transport(self) -> Any: ...
    @property
    def frame_writer(self) -> Callable[..., Any]: ...
    @property
    def on_inbound_frame(self) -> Callable[..., Any]: ...



from kombu.asynchronous.hub import Hub as Hub
from kombu.asynchronous.hub import get_event_loop as get_event_loop
from kombu.asynchronous.hub import set_event_loop as set_event_loop

__all__ = ("ERR", "READ", "WRITE", "Hub", "get_event_loop", "set_event_loop")

READ: int
WRITE: int
ERR: int



from typing import Any, ClassVar

from celery.bootsteps import Step
from typing_extensions import override

__all__ = ("Control",)

class Control(Step):
    name: ClassVar[str]  # pyright: ignore[reportIncompatibleVariableOverride]

    def __init__(self, c: Any, **kwargs: Any) -> None: ...
    @override
    def create(self, parent: Any) -> Any: ...
    def start(self, parent: Any) -> None: ...
    def stop(self, parent: Any) -> None: ...
    @override
    def include_if(self, c: Any) -> bool: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def info(self, obj: Any) -> dict[str, Any]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]



from collections.abc import Callable, Iterator
from datetime import datetime
from types import TracebackType
from typing import Any, NamedTuple
from zoneinfo import ZoneInfo

from typing_extensions import Self

__all__ = ("Entry", "Timer", "to_timestamp")

class scheduled(NamedTuple):
    eta: float
    priority: int
    entry: Entry

class Entry:
    fun: Callable[..., Any]
    args: list[Any]
    kwargs: dict[str, Any]
    tref: Any
    canceled: bool
    _last_run: float | None

    def __init__(
        self,
        fun: Callable[..., Any],
        args: list[Any] | None = ...,
        kwargs: dict[str, Any] | None = ...,
    ) -> None: ...
    def __call__(self) -> Any: ...
    def cancel(self) -> None: ...
    def __lt__(self, other: Entry) -> bool: ...
    def __le__(self, other: Entry) -> bool: ...
    def __gt__(self, other: Entry) -> bool: ...
    def __ge__(self, other: Entry) -> bool: ...
    @property
    def cancelled(self) -> bool: ...
    @cancelled.setter
    def cancelled(self, value: bool) -> None: ...

_Entry = Entry

class Timer:
    Entry: type[_Entry]
    max_interval: float
    on_error: Callable[[Exception], None] | None
    _queue: list[scheduled]

    def __init__(
        self,
        max_interval: float | None = ...,
        on_error: Callable[[Exception], None] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def __len__(self) -> int: ...
    def __nonzero__(self) -> bool: ...
    def __iter__(
        self,
        min: Callable[..., Any] = ...,
        nowfun: Callable[[], float] = ...,
        pop: Callable[..., Any] = ...,
        push: Callable[..., Any] = ...,
    ) -> Iterator[tuple[float | None, _Entry | None]]: ...
    def call_at(
        self,
        eta: float | datetime,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = ...,
        kwargs: dict[str, Any] | None = ...,
        priority: int = ...,
    ) -> _Entry: ...
    def call_after(
        self,
        secs: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = ...,
        kwargs: dict[str, Any] | None = ...,
        priority: int = ...,
    ) -> _Entry: ...
    def call_repeatedly(
        self,
        secs: float,
        fun: Callable[..., Any],
        args: tuple[Any, ...] = ...,
        kwargs: dict[str, Any] | None = ...,
        priority: int = ...,
    ) -> _Entry: ...
    def enter_at(
        self,
        entry: _Entry,
        eta: float | datetime | None = ...,
        priority: int = ...,
        time: Callable[[], float] = ...,
    ) -> _Entry | None: ...
    def enter_after(
        self,
        secs: float,
        entry: _Entry,
        priority: int = ...,
        time: Callable[[], float] = ...,
    ) -> _Entry: ...
    def apply_entry(self, entry: _Entry) -> None: ...
    def handle_error(self, exc_info: Exception) -> bool | None: ...
    def stop(self) -> None: ...
    def cancel(self, tref: _Entry) -> None: ...
    def clear(self) -> None: ...
    def _enter(
        self,
        eta: float,
        priority: int,
        entry: _Entry,
        push: Callable[..., Any] = ...,
    ) -> _Entry: ...
    @property
    def queue(self) -> list[scheduled]: ...
    @property
    def schedule(self) -> Timer: ...

def to_timestamp(
    d: datetime | float,
    default_timezone: ZoneInfo = ...,
    time: Callable[[], float] = ...,
) -> float: ...



from collections.abc import Callable
from typing import Any

from celery.backends.base import KeyValueStoreBackend

E_NO_PATH_SET: str
E_PATH_INVALID: str
E_PATH_NON_CONFORMING_SCHEME: str
default_encoding: str

class FilesystemBackend(KeyValueStoreBackend):
    def __init__(
        self,
        url: str | None = None,
        open: Callable[..., Any] = ...,
        unlink: Callable[[str], None] = ...,
        sep: str = "/",
        encoding: str = "UTF-8",
        *args: Any,
        **kwargs: Any,
    ) -> None: ...



from typing import Any, NamedTuple

class basic_return_t(NamedTuple):
    reply_code: int
    reply_text: str
    exchange: str
    routing_key: str
    message: Any

class queue_declare_ok_t(NamedTuple):
    queue: str
    message_count: int
    consumer_count: int



from collections.abc import Generator
from contextlib import contextmanager
from logging import LogRecord, handlers
from typing import Any

from celery import Celery
from celery.utils.dispatch.signal import Signal
from celery.worker.worker import WorkController
from typing_extensions import override

WORKER_LOGLEVEL: str

test_worker_starting: Signal
test_worker_started: Signal
test_worker_stopped: Signal

class TestWorkController(WorkController):
    __test__: bool
    logger_queue: Any
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

    class QueueHandler(handlers.QueueHandler):
        @override
        def prepare(self, record: LogRecord) -> LogRecord: ...
        @override
        def handleError(self, record: LogRecord) -> None: ...

    @override
    def start(self) -> Any: ...
    @override
    def on_consumer_ready(self, consumer: Any) -> None: ...
    def ensure_started(self) -> None: ...

@contextmanager
def start_worker(
    app: Celery,
    concurrency: int = 1,
    pool: str = "solo",
    loglevel: str | int = ...,
    logfile: str | None = None,
    perform_ping_check: bool = True,
    ping_task_timeout: float = 10.0,
    shutdown_timeout: float = 10.0,
    **kwargs: Any,
) -> Generator[WorkController, None, None]: ...
@contextmanager
def _start_worker_thread(
    app: Celery,
    concurrency: int = 1,
    pool: str = "solo",
    loglevel: str | int = ...,
    logfile: str | None = None,
    WorkController: type[TestWorkController] = ...,
    perform_ping_check: bool = True,
    shutdown_timeout: float = 10.0,
    **kwargs: Any,
) -> Generator[WorkController, None, None]: ...
@contextmanager
def _start_worker_process(
    app: Celery,
    concurrency: int = 1,
    pool: str = "solo",
    loglevel: str | int = ...,
    logfile: str | None = None,
    **kwargs: Any,
) -> Generator[None, None, None]: ...
def setup_app_for_worker(
    app: Celery,
    loglevel: str | int,
    logfile: str,
) -> None: ...



from collections.abc import Callable, Iterable, Mapping, Sequence
from typing import Any, Literal, TypeAlias, TypedDict

__all__ = ("Control", "Inspect", "flatten_reply")

from celery.app.base import Celery
from celery.result import _State
from kombu import Connection
from kombu.pidbox import Mailbox as KombuMailbox

_Reply: TypeAlias = Any

_Destination: TypeAlias = Sequence[str] | tuple[str, ...]

class _ReportReply(TypedDict):
    ok: str

class _ClockReply(TypedDict):
    clock: int

class _TaskInfo(TypedDict):
    id: str
    name: str
    args: list[Any]
    kwargs: dict[str, Any]
    type: str
    hostname: str
    time_start: float
    acknowledged: bool
    delivery_info: dict[str, Any]
    worker_pid: int

class _TaskScheduledInfo(TypedDict):
    eta: str
    priority: int
    request: _TaskInfo

class _StatInfo(TypedDict):
    broker: dict[str, Any]
    clock: int
    uptime: int
    pid: int
    pool: dict[str, Any]
    prefetch_count: int
    rusage: dict[str, Any]
    total: dict[str, int]

class _PingReply(TypedDict):
    ok: Literal["pong"]

class _QueueInfo(TypedDict):
    name: str
    exchange: dict[str, Any]
    routing_key: str
    queue_arguments: None | list[Any]
    binding_arguments: None | list[Any]
    consumer_arguments: None | list[Any]
    durable: bool
    exclusive: bool
    auto_delete: bool
    no_ack: bool
    alias: None | str
    bindings: list[Any]
    no_declare: bool
    expires: None | int
    message_ttl: None | int
    max_length: None | int
    max_length_bytes: None | int
    max_priority: None | int

def flatten_reply(reply: Iterable[Mapping[str, _Reply]]) -> dict[str, _Reply]: ...

class Inspect:
    app: Celery | None
    def __init__(
        self,
        destination: _Destination | None = ...,
        timeout: float = ...,
        callback: Callable[..., Any] | None = ...,
        connection: Connection | None = ...,
        app: Celery | None = ...,
        limit: int | None = ...,
        pattern: str | None = ...,
        matcher: Callable[..., Any] | None = ...,
    ) -> None: ...
    def _prepare(self, reply: bool) -> None | dict[str, _Reply]: ...
    def _request(
        self, command: str, **kwargs: Any
    ) -> None | list[dict[str, _Reply]]: ...
    def report(self) -> None | dict[str, _ReportReply]: ...
    def clock(self) -> None | dict[str, _ClockReply]: ...
    def active(self, safe: Any | None = ...) -> None | dict[str, list[_TaskInfo]]: ...
    def scheduled(
        self, safe: Any | None = ...
    ) -> None | dict[str, list[_TaskScheduledInfo]]: ...
    def reserved(self, safe: Any | None = ...) -> None | dict[str, list[_TaskInfo]]: ...
    def stats(self) -> None | dict[str, _StatInfo]: ...
    def revoked(self) -> None | dict[str, list[str]]: ...
    def registered(self, *taskinfoitems: Any) -> None | dict[str, list[str]]: ...
    registered_tasks = registered
    def ping(
        self, destination: _Destination | None = ...
    ) -> None | dict[str, _PingReply]: ...
    def active_queues(self) -> None | dict[str, list[_QueueInfo]]: ...
    def query_task(
        self, *ids: str
    ) -> None | dict[str, dict[str, tuple[_State, _TaskInfo]]]: ...
    def conf(self, with_defaults: bool = ...) -> None | dict[str, dict[str, Any]]: ...
    def hello(
        self, from_node: Any, revoked: Any | None = ...
    ) -> None | dict[str, _Reply]: ...
    def memsample(self) -> None | dict[str, _Reply]: ...
    def memdump(self, samples: int = ...) -> None | dict[str, _Reply]: ...
    def objgraph(
        self, type: str = ..., n: int = ..., max_depth: int = ...
    ) -> None | dict[str, dict[str, str]]: ...

class Control:
    Mailbox: type[KombuMailbox]
    app: Celery | None
    def __init__(self, app: Celery | None = ...) -> None: ...
    @property
    def inspect(self) -> Inspect: ...
    def purge(self, connection: Connection | None = ...) -> int: ...
    discard_all = purge
    def election(
        self,
        id: str,
        topic: str,
        action: str | None = ...,
        connection: Connection | None = ...,
    ) -> None: ...
    def revoke(
        self,
        task_id: str | Sequence[str],
        destination: _Destination | None = ...,
        terminate: bool = ...,
        signal: str = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def revoke_by_stamped_headers(
        self,
        headers: Mapping[str, str],
        destination: _Destination | None = ...,
        terminate: bool = ...,
        signal: str = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def terminate(
        self,
        task_id: str,
        destination: _Destination | None = ...,
        signal: str = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def ping(
        self,
        destination: _Destination | None = ...,
        timeout: float = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def rate_limit(
        self,
        task_name: str,
        rate_limit: int | str,
        destination: _Destination | None = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def add_consumer(
        self,
        queue: str,
        exchange: str | None = ...,
        exchange_type: str = ...,
        routing_key: str | None = ...,
        options: Mapping[str, Any] | None = ...,
        destination: _Destination | None = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def cancel_consumer(
        self, queue: str, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def time_limit(
        self,
        task_name: str,
        soft: float | None = ...,
        hard: float | None = ...,
        destination: _Destination | None = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def enable_events(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def disable_events(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def pool_grow(
        self, n: int = ..., destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def pool_shrink(
        self, n: int = ..., destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def autoscale(
        self, max: int, min: int, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def shutdown(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def pool_restart(
        self,
        modules: Sequence[str] | None = ...,
        reload: bool = ...,
        reloader: Callable[..., Any] | None = ...,
        destination: _Destination | None = ...,
        **kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...
    def heartbeat(
        self, destination: _Destination | None = ..., **kwargs: Any
    ) -> list[dict[str, _Reply]] | None: ...
    def broadcast(
        self,
        command: str,
        arguments: Mapping[str, Any] | None = ...,
        destination: _Destination | None = ...,
        connection: Connection | None = ...,
        reply: bool = ...,
        timeout: float = ...,
        limit: int | None = ...,
        callback: Callable[..., Any] | None = ...,
        channel: Any | None = ...,
        pattern: str | None = ...,
        matcher: Callable[..., Any] | None = ...,
        **extra_kwargs: Any,
    ) -> list[dict[str, _Reply]] | None: ...



from logging import Logger
from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property

logger: Logger
DEFAULT_PORT: int
DEFAULT_HOST: str

class LockError(Exception): ...

class Channel(VirtualChannel):
    prefix: str
    index: int | None
    timeout: str
    session_ttl: int

    @cached_property
    def lock_name(self) -> str: ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class Transport(VirtualTransport):
    Channel: type[Channel]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str



from typing import Any

from celery.backends.base import Backend
from celery.backends.database.models import Task, TaskSet

__all__ = ("DatabaseBackend",)

class DatabaseBackend(Backend):
    task_cls: type[Task]
    taskset_cls: type[TaskSet]

    @property
    def extended_result(self) -> bool: ...
    def ResultSession(self, session_manager: Any | None = None) -> Any: ...
    def __init__(
        self,
        dburi: str | None = None,
        engine_options: dict[str, Any] | None = None,
        url: str | None = None,
        **kwargs: Any,
    ) -> None: ...



from typing import Any

from kombu.asynchronous.aws.connection import AsyncAWSQueryConnection
from kombu.asynchronous.aws.sqs.queue import AsyncQueue
from typing_extensions import override
from vine import promise

__all__ = ("AsyncSQSConnection",)

class AsyncSQSConnection(AsyncAWSQueryConnection):
    message_system_attribute_names: list[str]
    message_attribute_names: list[str]

    def __init__(
        self,
        sqs_connection: Any,
        debug: int = ...,
        region: str | None = ...,
        message_system_attribute_names: list[str] | None = ...,
        message_attribute_names: list[str] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    @override
    def make_request(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self,
        operation_name: str,
        params: dict[str, Any],
        queue_url: str,
        verb: str,
        callback: Any = ...,
        protocol_params: dict[str, Any] | None = ...,
    ) -> promise: ...
    def create_queue(
        self, queue_name: str, visibility_timeout: int | None = ..., callback: Any = ...
    ) -> promise: ...
    def delete_queue(
        self, queue: AsyncQueue, force_deletion: bool = ..., callback: Any = ...
    ) -> promise: ...
    def get_queue_url(self, queue: str) -> str: ...
    def get_queue_attributes(
        self, queue: AsyncQueue, attribute: str = ..., callback: Any = ...
    ) -> promise: ...
    def set_queue_attribute(
        self, queue: AsyncQueue, attribute: str, value: Any, callback: Any = ...
    ) -> promise: ...
    def receive_message(
        self,
        queue: AsyncQueue,
        queue_url: str,
        number_messages: int = ...,
        visibility_timeout: int | None = ...,
        attributes: list[str] | None = ...,
        wait_time_seconds: int | None = ...,
        callback: Any = ...,
    ) -> promise: ...
    def delete_message(
        self, queue: str | AsyncQueue, receipt_handle: str, callback: Any = ...
    ) -> promise: ...
    def delete_message_batch(
        self, queue: AsyncQueue, messages: list[Any], callback: Any = ...
    ) -> promise: ...
    def delete_message_from_handle(
        self, queue: str | AsyncQueue, receipt_handle: str, callback: Any = ...
    ) -> promise: ...
    def send_message(
        self,
        queue: AsyncQueue,
        message_content: str,
        delay_seconds: int | None = ...,
        callback: Any = ...,
    ) -> promise: ...
    def send_message_batch(
        self,
        queue: AsyncQueue,
        messages: list[tuple[str, str, int]],
        callback: Any = ...,
    ) -> promise: ...
    def change_message_visibility(
        self,
        queue: AsyncQueue,
        receipt_handle: str,
        visibility_timeout: int,
        callback: Any = ...,
    ) -> promise: ...
    def change_message_visibility_batch(
        self, queue: AsyncQueue, messages: list[tuple[Any, int]], callback: Any = ...
    ) -> promise: ...
    def get_all_queues(self, prefix: str = ..., callback: Any = ...) -> promise: ...
    def get_queue(self, queue_name: str, callback: Any = ...) -> promise: ...
    def lookup(self, queue_name: str, callback: Any = ...) -> promise: ...
    def get_dead_letter_source_queues(
        self, queue: AsyncQueue, callback: Any = ...
    ) -> promise: ...
    def add_permission(
        self,
        queue: AsyncQueue,
        label: str,
        aws_account_id: str,
        action_name: str,
        callback: Any = ...,
    ) -> promise: ...
    def remove_permission(
        self, queue: AsyncQueue, label: str, callback: Any = ...
    ) -> promise: ...



from datetime import datetime
from typing import Any

from celery.result import AsyncResult
from kombu import Connection
from kombu.messaging import Producer

def send_task(
    name: str,
    args: tuple[Any, ...] | None = ...,
    kwargs: dict[str, Any] | None = ...,
    countdown: float | None = ...,
    eta: datetime | None = ...,
    task_id: str | None = ...,
    producer: Producer | None = ...,
    connection: Connection | None = ...,
    router: Any | None = ...,
    result_cls: type[AsyncResult[Any]] | None = ...,
    expires: float | datetime | None = ...,
    publisher: Any | None = ...,
    link: Any | None = ...,
    link_error: Any | None = ...,
    add_to_parent: bool = ...,
    group_id: str | None = ...,
    group_index: int | None = ...,
    retries: int = ...,
    chord: Any | None = ...,
    reply_to: str | None = ...,
    time_limit: float | None = ...,
    soft_time_limit: float | None = ...,
    root_id: str | None = ...,
    parent_id: str | None = ...,
    route_name: str | None = ...,
    shadow: str | None = ...,
    chain: Any | None = ...,
    task_type: Any | None = ...,
    replaced_task_nesting: int = ...,
    **options: Any,
) -> AsyncResult[Any]: ...

__all__ = ["send_task"]



from collections.abc import Callable, Sequence
from enum import Enum
from types import TracebackType
from typing import Any

from kombu.connection import Connection
from kombu.entity import Exchange, Queue
from kombu.message import Message
from kombu.transport.base import StdChannel
from typing_extensions import Self, override

__all__ = ("Consumer", "Exchange", "Producer", "Queue")

class Producer:
    channel: Connection | StdChannel | None
    exchange: Exchange | None
    routing_key: str
    serializer: str | None
    compression: str | None
    auto_declare: bool
    on_return: Callable[[Exception, Exchange | str, str, Message], None] | None

    def __init__(
        self,
        channel: Connection | StdChannel,
        exchange: Exchange | str | None = ...,
        routing_key: str | None = ...,
        serializer: str | None = ...,
        auto_declare: bool | None = ...,
        compression: str | None = ...,
        on_return: (
            Callable[[Exception, Exchange | str, str, Message], None] | None
        ) = ...,
    ) -> None: ...
    def declare(self) -> None: ...
    def maybe_declare(
        self, entity: Exchange | Queue, retry: bool = ..., **retry_policy: Any
    ) -> bool: ...
    def publish(
        self,
        body: Any,
        routing_key: str | None = ...,
        delivery_mode: Enum | int | None = ...,
        mandatory: bool = ...,
        immediate: bool = ...,
        priority: int = ...,
        content_type: str | None = ...,
        content_encoding: str | None = ...,
        serializer: str | None = ...,
        headers: dict[Any, Any] | None = ...,
        compression: str | None = ...,
        exchange: Exchange | str | None = ...,
        retry: bool = ...,
        retry_policy: dict[Any, Any] | None = ...,
        declare: Sequence[Exchange | Queue] | None = ...,
        expiration: float | None = ...,
        timeout: float | None = ...,
        confirm_timeout: float | None = ...,
        **properties: Any,
    ) -> None: ...
    def revive(self, channel: StdChannel) -> None: ...
    def close(self) -> None: ...
    def release(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def __reduce_args__(self) -> tuple[Any, ...]: ...
    @override
    def __reduce__(self) -> tuple[Any, ...]: ...
    def _get_channel(self) -> StdChannel: ...
    def _set_channel(self, channel: Connection | StdChannel) -> None: ...
    def _delivery_details(
        self,
        exchange: Exchange | str,
        delivery_mode: int | None = ...,
        maybe_delivery_mode: Callable[..., int | None] = ...,
        Exchange: type[Exchange] = ...,
    ) -> tuple[str, int | None]: ...
    def _prepare(
        self,
        body: Any,
        serializer: str | None = ...,
        content_type: str | None = ...,
        content_encoding: str | None = ...,
        compression: str | None = ...,
        headers: dict[str, Any] | None = ...,
    ) -> tuple[bytes, str, str]: ...
    def _publish(
        self,
        body: bytes,
        priority: int | None,
        content_type: str | None,
        content_encoding: str | None,
        headers: dict[str, Any] | None,
        properties: dict[str, Any],
        routing_key: str,
        mandatory: bool,
        immediate: bool,
        exchange: str,
        declare: list[Any],
        timeout: float | None = ...,
        confirm_timeout: float | None = ...,
        retry: bool = ...,
        retry_policy: dict[str, Any] | None = ...,
    ) -> None: ...
    @property
    def __connection__(self) -> Connection | None: ...
    @property
    def connection(self) -> Connection | None: ...

class Consumer:
    ContentDisallowed: type[Exception]

    channel: Connection | StdChannel | None
    queues: list[Queue]
    accept: set[str] | None
    no_ack: bool | None
    auto_declare: bool
    callbacks: list[Callable[[Any, Message], None]] | None
    on_decode_error: Callable[[Message, Exception], None] | None
    on_message: Callable[[Message], None] | None
    prefetch_count: int | None
    tag_prefix: str

    def __init__(
        self,
        channel: Connection | StdChannel,
        queues: Sequence[Queue] | Queue | None = ...,
        no_ack: bool | None = ...,
        auto_declare: bool | None = ...,
        callbacks: Sequence[Callable[[Any, Message], None]] | None = ...,
        on_decode_error: Callable[[Message, Exception], None] | None = ...,
        on_message: Callable[[Message], None] | None = ...,
        accept: Sequence[str] | None = ...,
        prefetch_count: int | None = ...,
        tag_prefix: str | None = ...,
    ) -> None: ...
    def declare(self) -> None: ...
    def register_callback(self, callback: Callable[[Any, Message], None]) -> None: ...
    def add_queue(self, queue: Queue) -> Self: ...
    def consume(self, no_ack: bool | None = ...) -> None: ...
    def cancel(self) -> None: ...
    close = cancel
    def cancel_by_queue(self, queue: str | Queue) -> None: ...
    def consuming_from(self, queue: str | Queue) -> bool: ...
    def purge(self) -> int: ...
    def flow(self, active: bool) -> None: ...
    def qos(
        self,
        prefetch_size: int = ...,
        prefetch_count: int = ...,
        apply_global: bool = ...,
    ) -> None: ...
    def recover(self, requeue: bool = ...) -> None: ...
    def receive(self, body: Any, message: Message) -> None: ...
    def revive(self, channel: StdChannel) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def _add_tag(self, queue: Queue, consumer_tag: str | None = ...) -> str: ...
    def _basic_consume(
        self,
        queue: Queue,
        consumer_tag: str | None = ...,
        no_ack: bool | None = ...,
        nowait: bool = ...,
    ) -> str: ...
    def _receive_callback(self, message: Message) -> None: ...
    @property
    def connection(self) -> Connection | None: ...



from abc import ABCMeta
from collections.abc import Callable
from typing import Any, NamedTuple

__all__ = ("Panel",)

class controller_info_t(NamedTuple):
    alias: str | None
    type: str
    visible: bool
    default_timeout: float
    help: str
    signature: str | None
    args: list[tuple[str, type]] | None  # type: ignore[valid-type]
    variadic: str | None

class Panel(metaclass=ABCMeta):
    data: dict[str, Callable[..., Any]]
    meta: dict[str, controller_info_t]

    @classmethod
    def register(
        cls,
        method: Callable[..., Any],
        name: str | None = None,
    ) -> Callable[..., Any]: ...

def control_command(
    **kwargs: Any,
) -> Callable[[Callable[..., Any]], Callable[..., Any]]: ...
def inspect_command(
    **kwargs: Any,
) -> Callable[[Callable[..., Any]], Callable[..., Any]]: ...



from collections.abc import Callable
from types import TracebackType
from typing import Any

from typing_extensions import override
from vine.abstract import Thenable

class promise(Thenable):
    args: tuple[Any, ...] | None
    kwargs: dict[str, Any] | None
    fun: Callable[..., Any] | None
    on_error: Callable[..., Any] | None
    weak: bool

    @property
    def listeners(self) -> list[Callable[..., Any]]: ...
    ignore_result: bool
    cancelled: bool
    ready: bool
    failed: bool
    value: Any
    reason: BaseException | None

    def __init__(
        self,
        fun: Callable[..., Any] | None = ...,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        callback: Callable[..., Any] | None = ...,
        on_error: Callable[..., Any] | None = ...,
        weak: bool = ...,
        ignore_result: bool = ...,
    ) -> None: ...
    @override
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    @override
    def then(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self,
        callback: Callable[..., Any],
        on_error: Callable[..., Any] | None = ...,
    ) -> promise: ...
    @override
    def throw(
        self,
        exc: BaseException | None = ...,
        tb: TracebackType | None = ...,
        propagate: bool = ...,
    ) -> None: ...
    def throw1(self, exc: BaseException | None = ...) -> None: ...
    @override
    def cancel(self) -> None: ...

__all__ = ["promise"]



import socket
import sys
from contextlib import AbstractContextManager
from pdb import Pdb
from types import FrameType, TracebackType
from typing import Any, Literal, TextIO

from typing_extensions import Self, override

__all__ = (
    "CELERY_RDB_HOST",
    "CELERY_RDB_PORT",
    "DEFAULT_PORT",
    "Rdb",
    "debugger",
    "set_trace",
)

DEFAULT_PORT: int = 6899

CELERY_RDB_HOST: str
CELERY_RDB_PORT: int

class Rdb(Pdb, AbstractContextManager[Rdb]):
    me: str = "Remote Debugger"

    def __init__(
        self,
        host: str = ...,
        port: int = ...,
        port_search_limit: int = 100,
        port_skew: int = 0,
        out: TextIO | Any = sys.stdout,
    ) -> None: ...
    def get_avail_port(
        self,
        host: str,
        port: int,
        search_limit: int = 100,
        skew: int = 0,
    ) -> tuple[socket.socket, int]: ...
    def say(self, m: str) -> None: ...
    @override
    def __enter__(self) -> Self: ...
    @override
    def __exit__(
        self,
        typ: type[BaseException] | None,
        exc: BaseException | None,
        tb: TracebackType | None,
    ) -> bool | None: ...
    @override
    def do_continue(self, arg: object) -> Literal[1]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]

    do_cont = do_continue  # type: ignore[assignment]
    do_c = do_continue  # type: ignore[assignment]

    @override
    def do_quit(self, arg: object) -> Literal[1]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]

    do_exit = do_quit  # type: ignore[assignment]
    do_q = do_quit  # type: ignore[assignment]

    @override
    def set_quit(self) -> None: ...

def debugger() -> Rdb: ...
def set_trace(frame: FrameType | None = None) -> None: ...



from typing import Any, ClassVar

from celery.bootsteps import Step
from typing_extensions import override

__all__ = ("Mingle",)

class Mingle(Step):
    name: ClassVar[str]  # pyright: ignore[reportIncompatibleVariableOverride]
    compatible_transports: set[str]

    def __init__(self, c: Any, without_mingle: bool = False, **kwargs: Any) -> None: ...
    @override
    def create(self, parent: Any) -> Any: ...
    def start(self, c: Any) -> None: ...
    def stop(self, parent: Any) -> None: ...
    @override
    def include_if(self, parent: Any) -> bool: ...
    @override
    def info(self, obj: Any) -> dict[str, Any]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def compatible_transport(self, app: Any) -> bool: ...
    def on_clock_event(self, c: Any, clock: Any) -> None: ...
    def on_node_reply(self, c: Any, nodename: str, reply: Any) -> None: ...
    def on_revoked_received(self, c: Any, revoked: Any) -> None: ...
    def send_hello(self, c: Any) -> None: ...
    def sync(self, c: Any) -> None: ...
    def sync_with_node(
        self, c: Any, clock: Any = None, revoked: Any = None, **kwargs: Any
    ) -> None: ...



from collections.abc import Callable, Generator
from typing import Any
from uuid import UUID

from kombu import pools as pools
from kombu.connection import Connection as Connection
from kombu.entity import Exchange as Exchange
from kombu.entity import Queue as Queue
from kombu.entity import binding as binding
from kombu.message import Message as Message
from kombu.messaging import Consumer as Consumer
from kombu.messaging import Producer as Producer
from kombu.pools import Connections as _Connections
from kombu.pools import Producers as _Producers

__all__ = (
    "BrokerConnection",
    "Connection",
    "Consumer",
    "Exchange",
    "Message",
    "Producer",
    "Queue",
    "binding",
    "connections",
    "disable_insecure_serializers",
    "enable_insecure_serializers",
    "eventloop",
    "parse_url",
    "producers",
    "uuid",
)

BrokerConnection = Connection

connections: _Connections
producers: _Producers

def parse_url(url: str) -> dict[str, Any]: ...
def eventloop(
    conn: Connection,
    limit: int | None = ...,
    timeout: float | None = ...,
    ignore_timeouts: bool = ...,
) -> Generator[Any, None, None]: ...
def uuid(_uuid: Callable[[], UUID] = ...) -> str: ...
def enable_insecure_serializers(choices: Any = ...) -> None: ...
def disable_insecure_serializers(allowed: Any = ...) -> None: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("CacheBackend",)

class CacheBackend(KeyValueStoreBackend):
    servers: list[str] | None
    supports_native_join: bool

    @property
    def client(self) -> Any: ...
    def __init__(
        self,
        app: Any,
        expires: float | None = None,
        backend: str | None = None,
        options: dict[str, Any] | None = None,
        url: str | None = None,
        **kwargs: Any,
    ) -> None: ...



from celery.worker.consumer.agent import Agent as Agent
from celery.worker.consumer.connection import Connection as Connection
from celery.worker.consumer.consumer import Consumer as Consumer
from celery.worker.consumer.control import Control as Control
from celery.worker.consumer.events import Events as Events
from celery.worker.consumer.gossip import Gossip as Gossip
from celery.worker.consumer.heart import Heart as Heart
from celery.worker.consumer.mingle import Mingle as Mingle
from celery.worker.consumer.tasks import Tasks as Tasks

__all__ = (
    "Agent",
    "Connection",
    "Consumer",
    "Control",
    "Events",
    "Gossip",
    "Heart",
    "Mingle",
    "Tasks",
)



from collections import UserDict
from collections.abc import Callable, Hashable, Iterable, Iterator
from typing import Any, TypeVar

from typing_extensions import override

__all__ = (
    "LRUCache",
    "dictfilter",
    "is_list",
    "lazy",
    "maybe_evaluate",
    "maybe_list",
    "memoize",
    "retry_over_time",
)

_T = TypeVar("_T")
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")

KEYWORD_MARK: object

class ChannelPromise:
    def __init__(self, contract: Any) -> None: ...
    def __call__(self) -> Any: ...

class LRUCache(UserDict[_KT, _VT]):
    limit: int | None

    def __init__(self, limit: int | None = ...) -> None: ...
    def incr(self, key: _KT, delta: int = ...) -> int: ...
    def iteritems(self) -> Iterator[tuple[_KT, _VT]]: ...
    def iterkeys(self) -> Iterator[_KT]: ...
    def itervalues(self) -> Iterator[_VT]: ...
    @override
    def popitem(self, last: bool = ...) -> tuple[_KT, _VT]: ...
    def _iterate_keys(self) -> Iterator[_KT]: ...
    def _iterate_items(self) -> Iterator[tuple[_KT, _VT]]: ...
    def __getstate__(self) -> dict[str, Any]: ...
    def __setstate__(self, state: dict[str, Any]) -> None: ...

def memoize(
    maxsize: int | None = ...,
    keyfun: Callable[..., Any] | None = ...,
    Cache: type[LRUCache[Any, Any] | dict[Any, Any]] = ...,
) -> Callable[[Callable[..., _T]], Callable[..., _T]]: ...

class lazy:
    def __init__(self, fun: Callable[..., Any], *args: Any, **kwargs: Any) -> None: ...
    def __call__(self) -> Any: ...
    def __deepcopy__(self, memo: dict[int, Any]) -> lazy: ...
    @override
    def __eq__(self, other: object) -> bool: ...
    @override
    def __ne__(self, other: object) -> bool: ...
    @override
    def __reduce__(self) -> tuple[Any, ...]: ...
    def evaluate(self) -> Any: ...

def maybe_evaluate(value: Any) -> Any: ...
def is_list(
    obj: Any,
    scalars: tuple[type[Any], ...] = ...,
    iters: tuple[type[Any], ...] = ...,
) -> bool: ...
def maybe_list(obj: Any, scalars: tuple[type[Any], ...] = ...) -> list[Any]: ...
def dictfilter(
    d: dict[Hashable, Any] | None = ..., **kw: Any
) -> dict[Hashable, Any]: ...
def retry_over_time(
    fun: Callable[..., _T],
    catch: type[BaseException] | tuple[type[BaseException], ...],
    args: Iterable[Any] | None = ...,
    kwargs: dict[str, Any] | None = ...,
    errback: Callable[[BaseException, float], None] | None = ...,
    max_retries: int | None = ...,
    interval_start: float = ...,
    interval_step: float = ...,
    interval_max: float = ...,
    callback: Callable[[], None] | None = ...,
    timeout: float | None = ...,
) -> _T: ...
def reprkwargs(kwargs: dict[str, Any], sep: str = ..., fmt: str = ...) -> str: ...
def reprcall(
    name: str,
    args: tuple[Any, ...] = ...,
    kwargs: dict[str, Any] | None = ...,
    sep: str = ...,
) -> str: ...
def accepts_argument(func: Callable[..., Any], argument_name: str) -> bool: ...
def fxrange(
    start: float = ...,
    stop: float | None = ...,
    step: float = ...,
    repeatlast: bool = ...,
) -> Iterator[float]: ...
def fxrangemax(
    start: float = ...,
    stop: float | None = ...,
    step: float = ...,
    max: float | None = ...,
) -> Iterator[float]: ...
def shufflecycle(it: Iterable[_T]) -> Iterator[_T]: ...

promise = lazy
maybe_promise = maybe_evaluate



from typing import Any

__all__ = ("TokenBucket",)

class TokenBucket:
    fill_rate: float | None
    capacity: int
    timestamp: float | None

    def __init__(self, fill_rate: float, capacity: int = ...) -> None: ...
    def add(self, item: Any) -> None: ...
    def pop(self) -> Any: ...
    def clear_pending(self) -> None: ...
    def can_consume(self, tokens: int = ...) -> bool: ...
    def expected_time(self, tokens: int = ...) -> float: ...



from collections.abc import Callable, Generator
from contextlib import contextmanager
from logging import Logger
from typing import Any, NamedTuple

import redis as redis_module
from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import QoS as VirtualQoS
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from redis.client import Pipeline as _RedisPipeline
from redis.client import PubSub as _RedisPubSub
from typing_extensions import override

logger: Logger
crit: Callable[..., None]
warning: Callable[..., None]

DEFAULT_PORT: int
DEFAULT_DB: int
DEFAULT_HEALTH_CHECK_INTERVAL: int
PRIORITY_STEPS: list[int]

class error_classes_t(NamedTuple):
    connection_errors: tuple[type[Exception], ...]
    channel_errors: tuple[type[Exception], ...]

def get_redis_error_classes() -> error_classes_t: ...
def get_redis_ConnectionError() -> type[Exception]: ...
def _after_fork_cleanup_channel(channel: Channel) -> None: ...

class MutexHeld(Exception): ...

@contextmanager
def Mutex(client: Any, name: str, expire: int) -> Generator[None, None, None]: ...

class GlobalKeyPrefixMixin:
    PREFIXED_SIMPLE_COMMANDS: list[str]
    PREFIXED_COMPLEX_COMMANDS: dict[str, dict[str, int | None]]
    global_keyprefix: str

    def _prefix_args(self, args: tuple[Any, ...] | list[Any]) -> list[Any]: ...
    def parse_response(
        self, connection: Any, command_name: str, **options: Any
    ) -> Any: ...
    def execute_command(self, *args: Any, **kwargs: Any) -> Any: ...
    def pipeline(
        self, transaction: bool = ..., shard_hint: Any | None = ...
    ) -> PrefixedRedisPipeline: ...

class PrefixedStrictRedis(GlobalKeyPrefixMixin, redis_module.Redis):
    global_keyprefix: str

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def pubsub(self, **kwargs: Any) -> PrefixedRedisPubSub: ...

class PrefixedRedisPipeline(GlobalKeyPrefixMixin, _RedisPipeline):
    global_keyprefix: str

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class PrefixedRedisPubSub(_RedisPubSub):
    PUBSUB_COMMANDS: tuple[str, ...]
    global_keyprefix: str

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def _prefix_args(self, args: tuple[Any, ...] | list[Any]) -> list[Any]: ...
    @override
    def parse_response(self, *args: Any, **kwargs: Any) -> Any: ...
    @override
    def execute_command(self, *args: Any, **kwargs: Any) -> Any: ...

class QoS(VirtualQoS):
    restore_at_shutdown: bool

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def append(self, message: Any, delivery_tag: str) -> None: ...
    @override
    def restore_unacked(self, client: Any | None = ...) -> None: ...
    @override
    def ack(self, delivery_tag: str) -> None: ...
    @override
    def reject(self, delivery_tag: str, requeue: bool = ...) -> None: ...
    @contextmanager
    def pipe_or_acquire(
        self, pipe: Any | None = ..., client: Any | None = ...
    ) -> Generator[Any, None, None]: ...
    @override
    def restore_visible(
        self, start: int = ..., num: int = ..., interval: int = ...
    ) -> None: ...
    def restore_by_tag(
        self, tag: str, client: Any | None = ..., leftmost: bool = ...
    ) -> None: ...
    @cached_property
    def unacked_key(self) -> str: ...
    @cached_property
    def unacked_index_key(self) -> str: ...
    @cached_property
    def unacked_mutex_key(self) -> str: ...
    @cached_property
    def unacked_mutex_expire(self) -> int: ...
    @cached_property
    def visibility_timeout(self) -> int: ...

class MultiChannelPoller:
    eventflags: int
    after_read: set[Any] | None
    poller: Any

    def __init__(self) -> None: ...
    def close(self) -> None: ...
    def add(self, channel: Channel) -> None: ...
    def discard(self, channel: Channel) -> None: ...
    def on_poll_start(self) -> None: ...
    def on_poll_init(self, poller: Any) -> Any: ...
    def maybe_restore_messages(self) -> Any: ...
    def maybe_check_subclient_health(self) -> None: ...
    def on_readable(self, fileno: int) -> None: ...
    def handle_event(
        self, fileno: int, event: int
    ) -> tuple[Any, MultiChannelPoller] | None: ...
    def get(
        self, callback: Callable[..., Any], timeout: float | None = ...
    ) -> None: ...
    @property
    def fds(self) -> dict[int, tuple[Channel, str]]: ...

_QoS = QoS

class Channel(VirtualChannel):
    QoS: type[_QoS]  # pyright: ignore[reportIncompatibleVariableOverride]

    supports_fanout: bool
    keyprefix_queue: str
    keyprefix_fanout: str
    sep: str
    ack_emulation: bool
    unacked_key: str
    unacked_index_key: str
    unacked_mutex_key: str
    unacked_mutex_expire: int
    unacked_restore_limit: int | None
    visibility_timeout: int
    priority_steps: list[int]
    socket_timeout: float | None
    socket_connect_timeout: float | None
    socket_keepalive: bool | None
    socket_keepalive_options: dict[str, Any] | None
    retry_on_timeout: bool | None
    max_connections: int
    health_check_interval: int
    client_name: str | None
    fanout_prefix: bool | str
    fanout_patterns: bool
    global_keyprefix: str
    queue_order_strategy: str

    connection_class: type[Any] | None
    connection_class_ssl: type[Any] | None

    from_transport_options: tuple[str, ...]

    handlers: dict[str, Callable[..., Any]]
    brpop_timeout: int
    active_fanout_queues: set[str]
    auto_delete_queues: set[str]
    Client: type[Any]
    ResponseError: type[Exception]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def basic_consume(self, queue: str, *args: Any, **kwargs: Any) -> Any: ...
    @override
    def basic_cancel(self, consumer_tag: str) -> Any: ...
    @override
    def close(self) -> None: ...
    @override
    def get_table(self, exchange: str) -> list[tuple[str, str, str]]: ...
    def priority(self, n: int) -> int: ...
    @contextmanager
    def conn_or_acquire(
        self, client: Any | None = ...
    ) -> Generator[Any, None, None]: ...
    @property
    def pool(self) -> Any: ...
    @property
    def async_pool(self) -> Any: ...
    @cached_property
    def client(self) -> Any: ...
    @cached_property
    def subclient(self) -> Any: ...
    @property
    def active_queues(self) -> set[str]: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

    polling_interval: None  # pyright: ignore[reportIncompatibleVariableOverride]
    brpop_timeout: int
    default_port: int  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    implements: Any
    connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    channel_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    cycle: MultiChannelPoller | None  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def driver_version(self) -> str: ...
    @override
    def register_with_event_loop(self, connection: Any, loop: Any) -> None: ...
    @override
    def on_readable(self, fileno: int) -> None: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]

class SentinelManagedSSLConnection:
    def __init__(self, **kwargs: Any) -> None: ...

class SentinelChannel(Channel):
    from_transport_options: tuple[str, ...]
    connection_class: type[Any] | None
    connection_class_ssl: type[Any] | None

class SentinelTransport(Transport):
    default_port: int
    Channel: type[SentinelChannel]  # pyright: ignore[reportIncompatibleVariableOverride]



from collections.abc import Callable, Iterable
from types import TracebackType
from typing import Any

from typing_extensions import Self

__all__ = ["Client", "Listener", "Pipe", "wait"]

from socket import socket

class _SocketContainer:
    def __init__(self, sock: socket) -> None: ...

class _ConnectionBase:
    def __init__(
        self, handle: int, readable: bool = ..., writable: bool = ...
    ) -> None: ...
    def __del__(self) -> None: ...
    def close(self) -> None: ...
    def send(self, obj: Any) -> None: ...
    def recv(self) -> Any: ...
    def poll(self, timeout: float = ...) -> bool: ...
    def send_bytes(
        self, buf: bytes, offset: int = ..., size: int | None = ...
    ) -> None: ...
    def recv_bytes(self, maxlength: int | None = ...) -> bytes: ...
    def recv_bytes_into(self, buf: bytearray, offset: int = ...) -> int: ...
    def send_offset(self, buf: bytes, offset: int) -> None: ...
    def setblocking(self, blocking: bool) -> None: ...
    @property
    def closed(self) -> bool: ...
    @property
    def readable(self) -> bool: ...
    @property
    def writable(self) -> bool: ...
    def fileno(self) -> int: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class Connection(_ConnectionBase): ...

class Listener:
    def __init__(
        self,
        address: str | tuple[str, int] | None = ...,
        family: str | None = ...,
        backlog: int = ...,
        authkey: bytes | None = ...,
    ) -> None: ...
    def accept(self) -> Connection: ...
    def close(self) -> None: ...
    @property
    def address(self) -> str | tuple[str, int]: ...
    @property
    def last_accepted(self) -> str | tuple[str, int] | None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class SocketListener:
    def __init__(
        self,
        address: str | tuple[str, int],
        family: str,
        backlog: int = ...,
    ) -> None: ...
    def accept(self) -> Connection: ...
    def close(self) -> None: ...

class ConnectionWrapper:
    def __init__(
        self,
        conn: Connection,
        dumps: Callable[..., bytes],
        loads: Callable[..., Any],
    ) -> None: ...
    def send(self, obj: Any) -> None: ...
    def recv(self) -> Any: ...

class XmlListener(Listener): ...

def Client(
    address: str | tuple[str, int],
    family: str | None = ...,
    authkey: bytes | None = ...,
) -> Connection: ...
def Pipe(
    duplex: bool = ..., rnonblock: bool = ..., wnonblock: bool = ...
) -> tuple[Connection, Connection]: ...
def wait(object_list: Iterable[Any], timeout: float | None = ...) -> list[Any]: ...



from types import TracebackType
from typing import Any

from amqp import ChannelError as ChannelError
from amqp import ConnectionError as ConnectionError

__all__ = (
    "ChannelError",
    "ChannelLimitExceeded",
    "ConnectionError",
    "ConnectionLimitExceeded",
    "DecodeError",
    "EncodeError",
    "HttpError",
    "InconsistencyError",
    "KombuError",
    "LimitExceeded",
    "MessageStateError",
    "NotBoundError",
    "OperationalError",
    "ResourceError",
    "SerializationError",
    "SerializerNotInstalled",
    "TimeoutError",
    "VersionMismatch",
    "reraise",
)

class KombuError(Exception): ...
class OperationalError(KombuError): ...
class SerializationError(KombuError): ...
class EncodeError(SerializationError): ...
class DecodeError(SerializationError): ...
class NotBoundError(KombuError): ...
class MessageStateError(KombuError): ...
class TimeoutError(KombuError): ...
class LimitExceeded(KombuError): ...
class ConnectionLimitExceeded(LimitExceeded): ...
class ChannelLimitExceeded(LimitExceeded): ...
class VersionMismatch(KombuError): ...
class SerializerNotInstalled(KombuError): ...
class ContentDisallowed(SerializerNotInstalled): ...

class ResourceError(KombuError):
    code: int

class InconsistencyError(ConnectionError): ...

class HttpError(Exception):
    code: int
    message: str | None
    response: Any | None

    def __init__(
        self, code: int, message: str | None = ..., response: Any | None = ...
    ) -> None: ...

def reraise(
    tp: type[BaseException] | None,
    value: BaseException | None,
    tb: TracebackType | None = ...,
) -> None: ...



from typing import Any, TextIO

default_encoding_file: TextIO | None

def bytes_to_str(s: bytes | str) -> str: ...
def str_to_bytes(s: str | bytes) -> bytes: ...
def ensure_bytes(s: str | bytes) -> bytes: ...
def from_utf8(s: str | bytes, *args: Any, **kwargs: Any) -> str: ...
def default_encode(obj: Any) -> bytes: ...
def default_encoding(file: TextIO | None = ...) -> str: ...
def get_default_encoding_file() -> TextIO | None: ...
def set_default_encoding_file(file: TextIO | None) -> None: ...
def safe_str(s: Any, errors: str = ...) -> str: ...
def safe_repr(o: Any, errors: str = ...) -> str: ...



from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from typing_extensions import override

CHARS_REPLACE_TABLE: dict[int, int]

class Channel(VirtualChannel):
    default_visibility_timeout: int
    domain_format: str
    _slmq: Any
    _queue_cache: dict[str, str]
    _noack_queues: set[str]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def basic_consume(
        self, queue: str, no_ack: bool, *args: Any, **kwargs: Any
    ) -> str: ...
    @override
    def basic_cancel(self, consumer_tag: str) -> None: ...
    def entity_name(self, name: str, table: dict[int, int] = ...) -> str: ...
    @override
    def basic_ack(self, delivery_tag: int) -> None: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def delete_message(self, queue: str, message_id: str) -> None: ...
    @property
    def visibility_timeout(self) -> int: ...
    @cached_property
    def queue_name_prefix(self) -> str: ...
    @property
    def slmq(self) -> Any: ...
    @property
    def conninfo(self) -> Any: ...
    @property
    def transport_options(self) -> dict[str, Any]: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    connection_errors: tuple[type[Exception] | None, ...]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]



from typing import Any, ClassVar

__all__ = ("Blueprint", "ConsumerStep", "StartStopStep", "Step")

from celery.utils.graph import GraphFormatter
from kombu import Consumer
from typing_extensions import override

class StepFormatter(GraphFormatter):
    blueprint_prefix: str
    conditional_prefix: str
    blueprint_scheme: dict[str, str]
    @override
    def label(self, step: Any) -> Any: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def node(self, obj: Any, **attrs: Any) -> Any: ...
    @override
    def edge(self, a: Any, b: Any, **attrs: Any) -> Any: ...

class Blueprint:
    GraphFormatter = StepFormatter
    name: ClassVar[str | None]
    state: Any
    started: int
    default_steps: set[Any]
    state_to_name: Any
    types: Any
    on_start: Any
    on_close: Any
    on_stopped: Any
    shutdown_complete: Any
    steps: Any
    def __init__(
        self,
        steps: Any | None = ...,
        name: Any | None = ...,
        on_start: Any | None = ...,
        on_close: Any | None = ...,
        on_stopped: Any | None = ...,
    ) -> None: ...
    def start(self, parent: Any) -> None: ...
    def human_state(self) -> str: ...
    def info(self, parent: Any) -> Any: ...
    def close(self, parent: Any) -> None: ...
    def restart(
        self,
        parent: Any,
        method: str = ...,
        description: str = ...,
        propagate: bool = ...,
    ) -> None: ...
    def send_all(
        self,
        parent: Any,
        method: Any,
        description: Any | None = ...,
        reverse: bool = ...,
        propagate: bool = ...,
        args: Any = ...,
    ) -> None: ...
    def stop(self, parent: Any, close: bool = ..., terminate: bool = ...) -> None: ...
    def join(self, timeout: Any | None = ...) -> None: ...
    def apply(self, parent: Any, **kwargs: Any) -> Any: ...
    def connect_with(self, other: Any) -> None: ...
    def __getitem__(self, name: str) -> Any: ...
    def claim_steps(self) -> Any: ...
    def load_step(self, step: Any) -> Any: ...
    @property
    def alias(self) -> Any: ...

class StepType(type):
    name: ClassVar[str | None]
    requires: tuple[type[Step] | str, ...] | None
    def __new__(cls, name: Any, bases: Any, attrs: Any) -> Any: ...

class Step(metaclass=StepType):
    name: ClassVar[str | None]
    label: ClassVar[str | None]
    conditional: bool
    requires: tuple[type[Step] | str, ...]
    last: bool
    enabled: bool
    def __init__(self, parent: Any, **kwargs: Any) -> None: ...
    def include_if(self, parent: Any) -> Any: ...
    def instantiate(self, name: Any, *args: Any, **kwargs: Any) -> Any: ...
    def include(self, parent: Any) -> Any: ...
    def create(self, parent: Any) -> None: ...
    @property
    def alias(self) -> Any: ...
    def info(self, obj: Any) -> None: ...

class StartStopStep(Step):
    obj: Any
    def start(self, parent: Any) -> Any: ...
    def stop(self, parent: Any) -> Any: ...
    def close(self, parent: Any) -> None: ...
    def terminate(self, parent: Any) -> Any: ...
    @override
    def include(self, parent: Any) -> Any: ...

class ConsumerStep(StartStopStep):
    consumers: list[Consumer] | None
    def get_consumers(self, channel: Any) -> list[Consumer]: ...
    @override
    def start(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self, c: Any
    ) -> None: ...
    @override
    def stop(  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
        self, c: Any
    ) -> None: ...
    def shutdown(self, c: Any) -> None: ...



from collections.abc import Callable, Generator, Iterable, Iterator
from contextlib import contextmanager
from types import TracebackType
from typing import Any, TypeVar

from kombu.messaging import Consumer as _Consumer
from kombu.messaging import Producer as _Producer
from kombu.simple import SimpleBuffer as _SimpleBuffer
from kombu.simple import SimpleQueue as _SimpleQueue
from kombu.transport.base import Management, StdChannel, Transport
from kombu.utils.objects import cached_property
from typing_extensions import Self, override

__all__ = ("ChannelPool", "Connection", "ConnectionPool")

def is_connection(obj: Any) -> bool: ...
def maybe_channel(channel: StdChannel | Connection | None) -> StdChannel | None: ...

_T = TypeVar("_T")

class Resource:
    LimitExceeded: type[Exception]
    close_after_fork: bool
    limit: int | None
    preload: int | None

    def __init__(
        self,
        limit: int | None = ...,
        preload: int | None = ...,
        close_after_fork: bool | None = ...,
    ) -> None: ...
    def setup(self) -> None: ...
    def acquire(self, block: bool = ..., timeout: float | None = ...) -> Any: ...
    def prepare(self, resource: Any) -> Any: ...
    def close_resource(self, resource: Any) -> None: ...
    def release_resource(self, resource: Any) -> None: ...
    def release(self, resource: Any) -> None: ...
    def collect_resource(self, resource: Any) -> None: ...
    def replace(self, resource: Any) -> None: ...
    def force_close_all(self, close_pool: bool = ...) -> None: ...
    def resize(
        self,
        limit: int,
        force: bool = ...,
        ignore_errors: bool = ...,
        reset: bool = ...,
    ) -> None: ...

class Connection:
    port: int | None
    virtual_host: str
    connect_timeout: float
    uri_prefix: str | None
    declared_entities: set[Any] | None
    cycle: Iterator[str] | None
    transport_options: dict[str, Any] | None
    failover_strategy: str | Callable[[Iterable[str]], Iterator[str]]
    heartbeat: float | None
    resolve_aliases: dict[str, str]
    failover_strategies: dict[str, Callable[[Iterable[str]], Iterator[str]]]
    hostname: str | None
    userid: str | None
    password: str | None
    ssl: bool | dict[str, Any] | None
    login_method: str | None
    alt: list[str]
    insist: bool
    transport_cls: str
    credential_provider: Any | None

    def __init__(
        self,
        hostname: str = ...,
        userid: str | None = ...,
        password: str | None = ...,
        virtual_host: str | None = ...,
        port: int | None = ...,
        insist: bool = ...,
        ssl: bool | dict[str, Any] = ...,
        transport: str | type[Transport] | None = ...,
        connect_timeout: float = ...,
        transport_options: dict[str, Any] | None = ...,
        login_method: str | None = ...,
        uri_prefix: str | None = ...,
        heartbeat: float = ...,
        failover_strategy: str | Callable[[Iterable[str]], Iterator[str]] = ...,
        alternates: list[str] | None = ...,
        credential_provider: Any | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def switch(self, conn_str: str) -> None: ...
    def maybe_switch_next(self) -> None: ...
    def register_with_event_loop(self, loop: Any) -> None: ...
    def connect(self) -> Self: ...
    def channel(self) -> StdChannel: ...
    def drain_events(self, **kwargs: Any) -> None: ...
    def maybe_close_channel(self, channel: StdChannel | None) -> None: ...
    def collect(self, socket_timeout: float | None = ...) -> None: ...
    def release(self) -> None: ...
    close = release
    def ensure_connection(
        self,
        errback: Callable[[Exception, float], None] | None = ...,
        max_retries: int = ...,
        interval_start: int = ...,
        interval_step: int = ...,
        interval_max: int = ...,
        callback: Callable[[], None] | None = ...,
        reraise_as_library_errors: bool = ...,
        timeout: int | None = ...,
    ) -> Self: ...
    def _ensure_connection(
        self,
        errback: Callable[[Exception, float], None] | None = ...,
        max_retries: int | None = ...,
        interval_start: int = ...,
        interval_step: int = ...,
        interval_max: int = ...,
        callback: Callable[[], None] | None = ...,
        reraise_as_library_errors: bool = ...,
        timeout: int | None = ...,
    ) -> Self: ...
    def autoretry(
        self,
        fun: Callable[..., _T],
        channel: StdChannel | None = ...,
        **ensure_options: Any,
    ) -> _T: ...
    def ensure(
        self,
        obj: Any,
        fun: Callable[..., _T],
        errback: Callable[[Exception, float], None] | None = ...,
        max_retries: int | None = ...,
        interval_start: int = ...,
        interval_step: int = ...,
        interval_max: int = ...,
        on_revive: Callable[[StdChannel], None] | None = ...,
        retry_errors: tuple[type[BaseException], ...] | None = ...,
    ) -> Callable[..., _T]: ...
    def revive(self, new_channel: StdChannel) -> None: ...
    def completes_cycle(self, retries: int) -> bool: ...
    def create_transport(self) -> Transport: ...
    def get_transport_cls(self) -> type[Transport]: ...
    def clone(self, **kwargs: Any) -> Connection: ...
    def get_heartbeat_interval(self) -> int: ...
    def heartbeat_check(self, rate: int = ...) -> None: ...
    def get_manager(self, *args: Any, **kwargs: Any) -> Management: ...
    def info(self) -> dict[str, Any]: ...
    def as_uri(
        self,
        include_password: bool = ...,
        mask: str = ...,
        getfields: Callable[..., Any] | None = ...,
    ) -> str: ...
    def Pool(self, limit: int | None = ..., **kwargs: Any) -> ConnectionPool: ...
    def ChannelPool(self, limit: int | None = ..., **kwargs: Any) -> ChannelPool: ...  # ty: ignore[invalid-type-form]
    def Producer(
        self, channel: StdChannel | None = ..., *args: Any, **kwargs: Any
    ) -> _Producer: ...
    def Consumer(
        self,
        queues: Any | None = ...,
        channel: StdChannel | None = ...,
        *args: Any,
        **kwargs: Any,
    ) -> _Consumer: ...
    def SimpleQueue(
        self,
        name: str,
        no_ack: bool | None = ...,
        queue_opts: dict[str, Any] | None = ...,
        queue_args: dict[str, Any] | None = ...,
        exchange_opts: dict[str, Any] | None = ...,
        channel: StdChannel | None = ...,
        **kwargs: Any,
    ) -> _SimpleQueue: ...
    def SimpleBuffer(
        self,
        name: str,
        no_ack: bool | None = ...,
        queue_opts: dict[str, Any] | None = ...,
        queue_args: dict[str, Any] | None = ...,
        exchange_opts: dict[str, Any] | None = ...,
        channel: StdChannel | None = ...,
        **kwargs: Any,
    ) -> _SimpleBuffer: ...
    def supports_exchange_type(self, exchange_type: str) -> bool: ...
    def _init_params(
        self,
        hostname: str,
        userid: str | None,
        password: str | None,
        virtual_host: str | None,
        port: int | None,
        insist: bool,
        ssl: bool | dict[str, Any],
        transport: str | type[Transport] | None,
        connect_timeout: float,
        login_method: str | None,
        heartbeat: float,
        credential_provider: Any | None,
    ) -> None: ...
    def _extract_failover_opts(self) -> dict[str, Any]: ...
    def _reraise_as_library_errors(
        self,
        ConnectionError: type[Exception] = ...,
        ChannelError: type[Exception] = ...,
    ) -> Generator[None, None, None]: ...
    def _debug(self, msg: str, *args: Any, **kwargs: Any) -> None: ...
    def _establish_connection(self) -> Any: ...
    def _connection_factory(self) -> Any: ...
    @contextmanager
    def _dummy_context(self) -> Generator[Self, None, None]: ...
    def _info(self, resolve: bool = ...) -> dict[str, Any]: ...
    def _do_close_self(self) -> None: ...
    def _do_close_transport(self) -> None: ...
    def _close(self) -> None: ...
    @override
    def __reduce__(self) -> tuple[Any, ...]: ...
    def __copy__(self) -> Connection: ...
    def __eqhash__(self) -> tuple[str, ...]: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    @property
    def connected(self) -> bool: ...
    @property
    def connection(self) -> Any: ...
    @cached_property
    def connection_errors(self) -> tuple[type[BaseException], ...]: ...
    @cached_property
    def channel_errors(self) -> tuple[type[BaseException], ...]: ...
    @cached_property
    def recoverable_connection_errors(self) -> tuple[type[BaseException], ...]: ...
    @cached_property
    def recoverable_channel_errors(self) -> tuple[type[BaseException], ...]: ...
    @property
    def transport(self) -> Transport: ...
    @property
    def default_channel(self) -> StdChannel: ...
    @property
    def host(self) -> str: ...
    @cached_property
    def manager(self) -> Management: ...
    @property
    def supports_heartbeats(self) -> bool: ...
    @property
    def is_evented(self) -> bool: ...
    @property
    def qos_semantics_matches_spec(self) -> bool: ...

class ConnectionPool(Resource):
    LimitExceeded: type[Exception]
    connection: Connection

    def __init__(
        self, connection: Connection, limit: int | None = ..., **kwargs: Any
    ) -> None: ...
    def new(self) -> Connection: ...
    @override
    def prepare(self, resource: Connection) -> Connection: ...
    @override
    def release_resource(self, resource: Connection) -> None: ...
    @override
    def close_resource(self, resource: Connection) -> None: ...
    @override
    def collect_resource(
        self, resource: Connection, socket_timeout: float | None = ...
    ) -> None: ...
    def acquire_channel(
        self, block: bool = ...
    ) -> Generator[StdChannel, None, None]: ...

class PooledConnection:
    pool: ConnectionPool

    def __init__(self, pool: ConnectionPool) -> None: ...
    def __enter__(self) -> Connection: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class ChannelPool(Resource):
    LimitExceeded: type[Exception]
    connection: Connection

    def __init__(
        self, connection: Connection, limit: int | None = ..., **kwargs: Any
    ) -> None: ...
    def new(self) -> StdChannel: ...
    @override
    def setup(self) -> None: ...
    @override
    def prepare(self, channel: StdChannel) -> StdChannel: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def release_resource(self, resource: StdChannel) -> None: ...
    @override
    def close_resource(self, resource: StdChannel) -> None: ...
